@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/config.c 1.18 1997/05/13 18:24:57 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/config.c 1.29 1997/09/21 14:22:07 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3.
   Configuration file management functions.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -16,10 +16,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 
 #include <unistd.h>
+#include <pwd.h>
+#include <grp.h>
 
 #include "config.h"
+#include "errors.h"
 #include "wwwoffle.h"
 
 
@@ -27,6 +31,10 @@
 #define SPOOL_DIR DEF_SPOOL
 #endif
 
+#ifndef CONF_DIR
+#define CONF_DIR DEF_CONF
+#endif
+
 /* Type definitions */
 
 /*+ The type of value to expect for a value. +*/
@@ -35,16 +43,23 @@ typedef enum _ConfigType
  Fixed,                         /*+ When the left hand side is fixed. +*/
  None,                          /*+ When there is no right hand side. +*/
 
+ Boolean,                       /*+ A boolean response (yes/no 1/0 true/false). +*/
+
  PortNumber,                    /*+ For port numbers (>0). +*/
  AgeDays,                       /*+ An age in days (can be -ve). +*/
  FileSize,                      /*+ A file size (must be >=0). +*/
- Boolean,                       /*+ A boolean response yes/no 1/0 true/false. +*/
+ CfgMaxServers,                 /*+ Max number of servers to fork (>0, <MAX_SERVERS). +*/
+ CfgMaxFetchServers,            /*+ Max number of servers for fetching pages (>0, <MAX_FETCH_SERVERS). +*/
+
+ UserId,                        /*+ For user IDs, (numeric or string). +*/
+ GroupId,                       /*+ For group IDs, (numeric or string). +*/
 
  String,                        /*+ For an arbitrary string (no whitespace). +*/
- DirName,                       /*+ For directory name values. +*/
+ DirName,                       /*+ For directory name values (string starting with '/'). +*/
  FileExt,                       /*+ A file extension (string). +*/
- HostAndPort,                   /*+ For host name and port numbers. +*/
- Host                           /*+ For host names. +*/
+ Host,                          /*+ For host names (string). +*/
+ HostAndPort,                   /*+ For host name and port numbers (string[:port]). +*/
+ CfgLogLevel                    /*+ A log level (debug, info, important, warning or fatal). +*/
 }
 ConfigType;
 
@@ -87,30 +102,44 @@ static KeyPair KeyPairEnd={NULL};
 /* StartUp section */
 
 /*+ The name of the configuration file. +*/
-char *ConfigFile=SPOOL_DIR "/wwwoffle.conf";
+char *ConfigFile=CONF_DIR "/wwwoffle.conf";
 
 /*+ The port number to use for the HTTP proxy port. +*/
-int HTTP_Port=8080;
+int HTTP_Port=DEF_HTTP_PORT;
 
 /*+ The port number to use for the wwwoffle port. +*/
-int WWWOFFLE_Port=8081;
+int WWWOFFLE_Port=DEF_WWWOFFLE_PORT;
 
 /*+ The spool directory. +*/
 char *SpoolDir=SPOOL_DIR;
 
+/*+ The user id for wwwoffled or -1 for none. +*/
+int WWWOFFLE_Uid=-1;
+
+/*+ The group id for wwwoffled or -1 for none. +*/
+int WWWOFFLE_Gid=-1;
+
 /*+ Whether to use the syslog facility or not. +*/
-int UseSyslog=0;
+int UseSyslog=1;
 
 /*+ The password required for demon configuration. +*/
 char *PassWord=NULL;
 
+/*+ Maximum number of servers  +*/
+int MaxServers=DEF_MAX_SERVERS; /*+ in total. +*/
+int MaxFetchServers=DEF_MAX_FETCH_SERVERS; /*+ for fetching. +*/
+
 /*+ The entries in the StartUp section. +*/
-static Entry startup_entries[]={{"http-port"    ,(void*)&HTTP_Port    ,0,Fixed,PortNumber},
-                                {"wwwoffle-port",(void*)&WWWOFFLE_Port,0,Fixed,PortNumber},
-                                {"spool-dir"    ,(void*)&SpoolDir     ,0,Fixed,DirName   },
-                                {"use-syslog"   ,(void*)&UseSyslog    ,0,Fixed,Boolean   },
-                                {"password"     ,(void*)&PassWord     ,0,Fixed,String    },
-                                {NULL           ,NULL                 ,0,   -1,        -1}};
+static Entry startup_entries[]={{"http-port"         ,(void*)&HTTP_Port       ,0,Fixed,PortNumber        },
+                                {"wwwoffle-port"     ,(void*)&WWWOFFLE_Port   ,0,Fixed,PortNumber        },
+                                {"spool-dir"         ,(void*)&SpoolDir        ,0,Fixed,DirName           },
+                                {"run-uid"           ,(void*)&WWWOFFLE_Uid    ,0,Fixed,UserId            },
+                                {"run-gid"           ,(void*)&WWWOFFLE_Gid    ,0,Fixed,GroupId           },
+                                {"use-syslog"        ,(void*)&UseSyslog       ,0,Fixed,Boolean           },
+                                {"password"          ,(void*)&PassWord        ,0,Fixed,String            },
+                                {"max-servers"       ,(void*)&MaxServers      ,0,Fixed,CfgMaxServers     },
+                                {"max-fetch-servers" ,(void*)&MaxFetchServers ,0,Fixed,CfgMaxFetchServers},
+                                {NULL                ,NULL                    ,0,   -1,                -1}};
 
 /*+ The StartUp section. +*/
 static Section startup_section={"StartUp",startup_entries};
@@ -118,9 +147,15 @@ static Section startup_section={"StartUp",startup_entries};
 
 /* Options Section */
 
+/*+ The level of error logging, see ErrorLevel in errors.h +*/
+int LogLevel=Important;
+
 /*+ The option to also fetch images. +*/
 int FetchImages=0;
 
+/*+ The option to also fetch frames. +*/
+int FetchFrames=0;
+
 /*+ The number of days to display in the index of the latest pages. +*/
 int IndexLatestDays=7;
 
@@ -130,12 +165,18 @@ int AddInfoRefresh=0;
 /*+ The option to always request the server for changes when it is the cache and online. +*/
 int RequestChanged=1;
 
+/*+ The option to allow or ignore the 'Pragma: no-cache' request. +*/
+int PragmaNoCache=1;
+
 /*+ The entries in the Options section. +*/
-static Entry options_entries[]={{"fetch-images"     ,(void*)&FetchImages    ,0,Fixed,Boolean},
-                                {"index-latest-days",(void*)&IndexLatestDays,0,Fixed,AgeDays},
-                                {"add-info-refresh" ,(void*)&AddInfoRefresh ,0,Fixed,Boolean},
-                                {"request-changed"  ,(void*)&RequestChanged ,0,Fixed,Boolean},
-                                {NULL               ,NULL                   ,0,   -1,     -1}};
+static Entry options_entries[]={{"log-level"        ,(void*)&LogLevel       ,0,Fixed,CfgLogLevel},
+                                {"fetch-images"     ,(void*)&FetchImages    ,0,Fixed,Boolean    },
+                                {"fetch-frames"     ,(void*)&FetchFrames    ,0,Fixed,Boolean    },
+                                {"index-latest-days",(void*)&IndexLatestDays,0,Fixed,AgeDays    },
+                                {"add-info-refresh" ,(void*)&AddInfoRefresh ,0,Fixed,Boolean    },
+                                {"request-changed"  ,(void*)&RequestChanged ,0,Fixed,Boolean    },
+                                {"pragma-no-cache"  ,(void*)&PragmaNoCache  ,0,Fixed,Boolean    },
+                                {NULL               ,NULL                   ,0,   -1,         -1}};
 
 /*+ The Options section. +*/
 static Section options_section={"Options",options_entries};
@@ -201,6 +242,27 @@ static Entry dontcache_entries[]={{"host"    ,(void*)&DontCacheHosts   ,1,Fixed,
 static Section dontcache_section={"DontCache",dontcache_entries};
 
 
+/* DontGetRecursive section */
+
+/*+ The list of hosts not to get. +*/
+static KeyPair **DontGetRecursiveHosts=NULL;
+
+/*+ The list of file extensions not to get. +*/
+static KeyPair **DontGetRecursiveExts=NULL;
+
+/*+ The list of non-got hostnames + pathnames. +*/
+static KeyPair **DontGetRecursiveHostPath=NULL;
+
+/*+ The entries in the DontGetRecursive section. +*/
+static Entry dontgetrecursive_entries[]={{"host"    ,(void*)&DontGetRecursiveHosts   ,1,Fixed,Host   },
+                                         {"file-ext",(void*)&DontGetRecursiveExts    ,1,Fixed,FileExt},
+                                         {""        ,(void*)&DontGetRecursiveHostPath,1,Host ,DirName},
+                                         {NULL      ,NULL                            ,0,   -1,     -1}};
+
+/*+ The DontGetRecursive section. +*/
+static Section dontgetrecursive_section={"DontGetRecursive",dontgetrecursive_entries};
+
+
 /* DontGet section */
 
 /*+ The list of hosts not to get. +*/
@@ -286,6 +348,7 @@ static Section *sections[]={&startup_section,
                             &localnet_section,
                             &allowedconnect_section,
                             &dontcache_section,
+                            &dontgetrecursive_section,
                             &dontget_section,
                             &censorheader_section,
                             &proxy_section,
@@ -427,7 +490,7 @@ int IsLocalNetHost(char *host)
 {
  KeyPair **p;
  char *colon=strchr(host,':');
- int isip=(*host>='0' && *host<='9')?1:0;
+ int isip=isdigit(*host);
  int isit=0;
 
  if(colon)
@@ -463,7 +526,7 @@ int IsLocalNetHost(char *host)
 int IsAllowedConnect(char *host)
 {
  KeyPair **p;
- int isip=(*host>='0' && *host<='9')?1:0;
+ int isip=isdigit(*host);
  int isit=0;
 
  if(LocalHost)
@@ -498,6 +561,22 @@ int IsNotCached(char *host,char *path)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Check if the specified URL is to be got recursively.
+
+  int IsNotGotRecursive Return true if it is not to be got recursively.
+
+  char *host The name of the host (and port number) to be checked.
+
+  char *path The path on the host to be checked.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+int IsNotGotRecursive(char *host,char *path)
+{
+ return(NotCachedOrGot(host,path,DontGetRecursiveHosts,DontGetRecursiveHostPath,DontGetRecursiveExts));
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Check if the specified URL is to be got.
 
@@ -538,15 +617,15 @@ static int NotCachedOrGot(char *host,char *path,KeyPair **Hosts,KeyPair **HostPa
  if(Hosts)
    {
     char *colon=strchr(host,':');
-    int isip=(*host>='0' && *host<='9')?1:0;
+    int isip=isdigit(*host);
 
     if(colon)
        *colon=0;
 
     for(p=Hosts;(*p)->name;p++)
-       if(strlen((*p)->name)<=strlen(host) &&
-          ((!isip && !strcmp ((*p)->name,host+strlen(host)-strlen((*p)->name))) ||
-           ( isip && !strncmp((*p)->name,host,strlen((*p)->name)))))
+       if(strlen((*p)->value.string)<=strlen(host) &&
+          ((!isip && !strcmp ((*p)->value.string,host+strlen(host)-strlen((*p)->value.string))) ||
+           ( isip && !strncmp((*p)->value.string,host,strlen((*p)->value.string)))))
          {isit=1;break;}
 
     if(colon)
@@ -556,7 +635,7 @@ static int NotCachedOrGot(char *host,char *path,KeyPair **Hosts,KeyPair **HostPa
  if(HostPath && !isit)
    {
     char *colon=strchr(host,':');
-    int isip=(*host>='0' && *host<='9')?1:0;
+    int isip=isdigit(*host);
 
     if(colon)
        *colon=0;
@@ -630,7 +709,7 @@ char *WhichProxy(char *host)
 {
  KeyPair **p;
  char *colon=strchr(host,':');
- int isip=(*host>='0' && *host<='9')?1:0;
+ int isip=isdigit(*host);
  char *proxy=DefaultProxy;
  int matchlen=0;
 
@@ -667,7 +746,7 @@ int WhatPurgeAge(char *host)
 {
  KeyPair **p;
  char *colon=strchr(host,':');
- int isip=(*host>='0' && *host<='9')?1:0;
+ int isip=isdigit(*host);
  int age=DefaultPurgeAge;
  int matchlen=0;
 
@@ -949,12 +1028,26 @@ static int ParseValue(char *text,ConfigType type,void *value,int rhs)
 {
  switch(type)
    {
+   case Fixed:
+   case None:
+    break;
+
+   case Boolean:
+    if(!*text)
+       *(int*)value=0;
+    else
+       if(!strcasecmp(text,"yes") || !strcasecmp(text,"true") || !strcmp(text,"1"))
+          *(int*)value=1;
+       else
+          *(int*)value=0;
+    break;
+
    case PortNumber:
     if(!*text)
       {errmsg=(char*)malloc(48);strcpy(errmsg,"Expecting a port number, got nothing.");break;}
-    if(atoi(text)<=0 || atoi(text)>65535)
-      {errmsg=(char*)malloc(32);sprintf(errmsg,"Invalid port number %d.",atoi(text));break;}
     *(int*)value=atoi(text);
+    if(*(int*)value<=0 || *(int*)value>65535)
+      {errmsg=(char*)malloc(32);sprintf(errmsg,"Invalid port number %d.",*(int*)value);break;}
     break;
 
    case AgeDays:
@@ -966,19 +1059,65 @@ static int ParseValue(char *text,ConfigType type,void *value,int rhs)
    case FileSize:
     if(!*text)
       {errmsg=(char*)malloc(48);strcpy(errmsg,"Expecting a file size in MB, got nothing.");break;}
-    if(atoi(text)<0)
-      {errmsg=(char*)malloc(48);sprintf(errmsg,"Invalid file size %d.",atoi(text));break;}
     *(int*)value=atoi(text);
+    if(*(int*)value<0)
+      {errmsg=(char*)malloc(48);sprintf(errmsg,"Invalid file size %d.",*(int*)value);break;}
     break;
 
-   case Boolean:
+   case CfgMaxServers:
     if(!*text)
-       *(int*)value=0;
+      {errmsg=(char*)malloc(60);strcpy(errmsg,"Expecting a maximum server count, got nothing.");break;}
+    *(int*)value=atoi(text);
+    if(*(int*)value<=0 || *(int*)value>MAX_SERVERS)
+      {errmsg=(char*)malloc(48);sprintf(errmsg,"Invalid maximum server count: %d.",*(int*)value);break;}
+    break;
+
+   case CfgMaxFetchServers:
+    if(!*text)
+      {errmsg=(char*)malloc(60);strcpy(errmsg,"Expecting a maximum fetch server count, got nothing.");break;}
+    *(int*)value=atoi(text);
+    if(*(int*)value<=0 || *(int*)value>MAX_FETCH_SERVERS)
+      {errmsg=(char*)malloc(48);sprintf(errmsg,"Invalid maximum fetch server count: %d.",*(int*)value);break;}
+    break;
+
+   case UserId:
+    if(!*text)
+      {errmsg=(char*)malloc(48);strcpy(errmsg,"Expecting a username or uid, got nothing.");break;}
     else
-       if(!strcasecmp(text,"yes") || !strcasecmp(text,"true") || !strcmp(text,"1"))
-          *(int*)value=1;
+      {
+       int uid;
+       struct passwd *userInfo = getpwnam(text);
+       if(userInfo)
+          uid = userInfo->pw_uid;
        else
-          *(int*)value=0;
+         {
+          if(sscanf(text, "%d", &uid) != 1)
+            {errmsg=(char*)malloc(32+strlen(text));sprintf(errmsg,"Invalid user %s.",text);break;}
+          if(!getpwuid(uid))
+            {errmsg=(char*)malloc(32);sprintf(errmsg,"Unknown user id %d.",uid);break;}
+         }
+       *(int*)value=uid;
+      }
+    break;
+
+   case GroupId:
+    if(!*text)
+      {errmsg=(char*)malloc(48);strcpy(errmsg,"Expecting a group name or gid, got nothing.");break;}
+    else
+      {
+       int gid;
+       struct group *groupInfo = getgrnam(text);
+       if(groupInfo)
+          gid = groupInfo->gr_gid;
+       else
+         {
+          if(sscanf(text, "%d", &gid) != 1)
+            {errmsg=(char*)malloc(32+strlen(text));sprintf(errmsg,"Invalid group %s.",text);break;}
+          if(!getgrgid(gid))
+            {errmsg=(char*)malloc(32);sprintf(errmsg,"Unknown group id %d.",gid);break;}
+         }
+       *(int*)value=gid;
+      }
     break;
 
    case String:
@@ -1037,9 +1176,23 @@ static int ParseValue(char *text,ConfigType type,void *value,int rhs)
       }
     break;
 
-   case Fixed:
-   case None:
-    ;
+   case CfgLogLevel:
+    if(!*text)
+      {errmsg=(char*)malloc(48);strcpy(errmsg,"Expecting a log level, got nothing.");break;}
+    if(strcasecmp(text,"debug")==0)
+       {*(int*)value=Debug;break;}
+    if(strcasecmp(text,"info")==0)
+       {*(int*)value=Inform;break;}
+    if(strcasecmp(text,"important")==0)
+       {*(int*)value=Important;break;}
+    if(strcasecmp(text,"warning")==0)
+       {*(int*)value=Warning;break;}
+    if(strcasecmp(text,"fatal")==0)
+       {*(int*)value=Fatal;break;}
+    /* Unrecognized value */
+    errmsg=(char*)malloc(48);
+    strcpy(errmsg,"Invalid value for log level.");
+    break;
    }
 
  if(errmsg)
@@ -1049,9 +1202,15 @@ static int ParseValue(char *text,ConfigType type,void *value,int rhs)
 }
 
 
+/*+ The old value of the level of error logging, see ErrorLevel in errors.h +*/
+static int old_LogLevel=Important;
+
 /*+ The old value of the option to also fetch images. +*/
 static int old_FetchImages=0;
 
+/*+ The old value of the option to also fetch frames. +*/
+static int old_FetchFrames=0;
+
 /*+ The old value of the number of days to display in the index of the latest pages. +*/
 static int old_IndexLatestDays=0;
 
@@ -1061,6 +1220,9 @@ static int old_AddInfoRefresh=0;
 /*+ The old value of the option to always request the server for changes when it is the cache and online. +*/
 static int old_RequestChanged=1;
 
+/*+ The old value of the option to allow or ignore the 'Pragma: no-cache' request. +*/
+static int old_PragmaNoCache=1;
+
 /*+ The old values of the list of localhost hostnames. +*/
 static KeyPair **old_LocalHost=NULL;
 
@@ -1079,6 +1241,15 @@ static KeyPair **old_DontCacheExts=NULL;
 /*+ The old value of the list of non-cached hostnames. +*/
 static KeyPair **old_DontCacheHostPath=NULL;
 
+/*+ The old value of the list of hosts not to get. +*/
+static KeyPair **old_DontGetRecursiveHosts=NULL;
+
+/*+ The old value of the list of file extensions not to get. +*/
+static KeyPair **old_DontGetRecursiveExts=NULL;
+
+/*+ The old value of the list of non-got hostnames. +*/
+static KeyPair **old_DontGetRecursiveHostPath=NULL;
+
 /*+ The old value of the list of hosts not to get. +*/
 static KeyPair **old_DontGetHosts=NULL;
 
@@ -1113,14 +1284,20 @@ static KeyPair **old_PurgeAges=NULL;
 
 static void SaveOldValues(void)
 {
+ old_LogLevel=LogLevel;
+ LogLevel=Important;
  old_FetchImages=FetchImages;
  FetchImages=0;
+ old_FetchFrames=FetchFrames;
+ FetchFrames=0;
  old_IndexLatestDays=IndexLatestDays;
  IndexLatestDays=7;
  old_AddInfoRefresh=AddInfoRefresh;
  AddInfoRefresh=0;
  old_RequestChanged=RequestChanged;
  RequestChanged=1;
+ old_PragmaNoCache=PragmaNoCache;
+ PragmaNoCache=1;
 
  old_LocalHost=LocalHost;
  LocalHost=NULL;
@@ -1138,6 +1315,13 @@ static void SaveOldValues(void)
  old_DontCacheHostPath=DontCacheHostPath;
  DontCacheHostPath=NULL;
 
+ old_DontGetRecursiveHosts=DontGetRecursiveHosts;
+ DontGetRecursiveHosts=NULL;
+ old_DontGetRecursiveExts=DontGetRecursiveExts;
+ DontGetRecursiveExts=NULL;
+ old_DontGetRecursiveHostPath=DontGetRecursiveHostPath;
+ DontGetRecursiveHostPath=NULL;
+
  old_DontGetHosts=DontGetHosts;
  DontGetHosts=NULL;
  old_DontGetExts=DontGetExts;
@@ -1167,10 +1351,13 @@ static void SaveOldValues(void)
 
 static void RestoreOldValues(void)
 {
+ LogLevel=old_LogLevel;
  FetchImages=old_FetchImages;
+ FetchFrames=old_FetchFrames;
  IndexLatestDays=old_IndexLatestDays;
  AddInfoRefresh=old_AddInfoRefresh;
  RequestChanged=old_RequestChanged;
+ PragmaNoCache=old_PragmaNoCache;
 
  if(LocalHost)
     FreeKeyPairList(LocalHost,1);
@@ -1194,6 +1381,16 @@ static void RestoreOldValues(void)
     FreeKeyPairList(DontCacheHostPath,3);
  DontCacheHostPath=old_DontCacheHostPath;
 
+ if(DontGetRecursiveHosts)
+    FreeKeyPairList(DontGetRecursiveHosts,2);
+ DontGetRecursiveHosts=old_DontGetRecursiveHosts;
+ if(DontGetRecursiveExts)
+    FreeKeyPairList(DontGetRecursiveExts,2);
+ DontGetRecursiveExts=old_DontGetRecursiveExts;
+ if(DontGetRecursiveHostPath)
+    FreeKeyPairList(DontGetRecursiveHostPath,3);
+ DontGetRecursiveHostPath=old_DontGetRecursiveHostPath;
+
  if(DontGetHosts)
     FreeKeyPairList(DontGetHosts,2);
  DontGetHosts=old_DontGetHosts;
@@ -1242,6 +1439,13 @@ static void RemoveOldValues(void)
  if(old_DontCacheHostPath)
     FreeKeyPairList(old_DontCacheHostPath,3);
 
+ if(old_DontGetRecursiveHosts)
+    FreeKeyPairList(old_DontGetRecursiveHosts,2);
+ if(old_DontGetRecursiveExts)
+    FreeKeyPairList(old_DontGetRecursiveExts,2);
+ if(old_DontGetRecursiveHostPath)
+    FreeKeyPairList(old_DontGetRecursiveHostPath,3);
+
  if(old_DontGetHosts)
     FreeKeyPairList(old_DontGetHosts,2);
  if(old_DontGetExts)
