@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/connect.c 1.14 1997/03/18 20:03:15 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/connect.c 1.18 1997/08/23 15:56:57 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2e.
   Handle WWWOFFLE connections received by the demon.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -38,11 +38,14 @@ extern int online;
 extern int n_servers,           /*+ in total. +*/
            n_fetch_servers;     /*+ fetching a previously requested page. +*/
 
+/*+ The wwwoffle client file descriptor when fetching. +*/
+extern int fetch_fd;
+
 /*+ The pids of the servers that are fetching. +*/
 extern int fetch_pids[MAX_FETCH_SERVERS];
 
-/*+ The wwwoffle client file descriptor when fetching. +*/
-extern int fetch_fd;
+/*+ The current status, fetching or not. +*/
+extern int fetching;
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -126,15 +129,12 @@ void CommandConnect(FILE *client)
        PrintMessage(Important,"WWWOFFLE Offline.");
 
     online=0;
-
-    if(fetch_fd>0)
-       fetch_fd=-fetch_fd;
    }
  else if(!strncmp(&line[9],"FETCH",5))
    {
     char *msg;
 
-    if(fetch_fd)
+    if(fetch_fd!=-1)
        msg="WWWOFFLE Already fetching.\n";
     else if(!online)
        msg="WWWOFFLE Must be online to fetch.\n";
@@ -145,11 +145,12 @@ void CommandConnect(FILE *client)
 
     PrintMessage(Important,"WWWOFFLE Fetch.");
 
-    if(!fetch_fd && online)
+    if(fetch_fd==-1 && online)
       {
        fetch_fd=fileno(client);
+       fetching=1;
 
-       for(;n_fetch_servers<MAX_FETCH_SERVERS && n_servers<MAX_SERVERS;)
+       while(n_fetch_servers<MaxFetchServers && n_servers<MaxServers)
           ForkServer(fetch_fd,0);
       }
    }
@@ -234,7 +235,7 @@ void ForkServer(int client,int browser)
          {
           int i;
 
-          for(i=0;i<MAX_FETCH_SERVERS;i++)
+          for(i=0;i<MaxFetchServers;i++)
              if(fetch_pids[i]==0)
                {fetch_pids[i]=pid;break;}
 
@@ -247,7 +248,7 @@ void ForkServer(int client,int browser)
       {
        int status;
 
-       if(fetch_fd && fetch_fd!=client)
+       if(fetch_fd!=-1 && fetch_fd!=client)
           close(fetch_fd);
 
        close(http_fd);
