@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/errors.h 1.6 1997/02/20 19:58:42 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/errors.h 1.8 1997/08/23 15:56:58 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.1.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2e.
   Error logging header file.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -16,9 +16,6 @@
 #ifndef ERRORS_H
 #define ERRORS_H    /*+ To stop multiple inclusions. +*/
 
-/*+ For debugging messages set to 1, else leave at 0. +*/
-#define DEBUG 0
-
 typedef enum _ErrorLevel
 {
  Debug,                         /*+ For debugging, only if DEBUG is defined (not in syslog). +*/
@@ -31,7 +28,7 @@ ErrorLevel;
 
 /* In errors.c */
 
-void InitErrorHandler(char *name,int syslogable);
+void InitErrorHandler(char *name,int syslogable,int stderrable);
 void PrintMessage(ErrorLevel errlev,char* fmt, ...);
 
 #endif /* ERRORS_H */
