@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/index.c 1.19 1997/06/23 20:16:14 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/index.c 1.27 1997/10/06 17:57:02 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3a.
   Generate an index of the web pages that are cached in the system.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -17,10 +17,10 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <time.h>
 #include <utime.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <time.h>
 #include <dirent.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -33,9 +33,10 @@
 /*+ A type to contain the information required to sort the files. +*/
 typedef struct _FileIndex
 {
- char *name;                    /*+ The name of the file. +*/
- char *type;                    /*+ The type (file extension) of the file. +*/
- long time;                     /*+ The time of the file, +*/
+ char *name;                    /*+ The URL of the file. +*/
+ char *file;                    /*+ The name of the file. +*/
+ char *type;                    /*+ The type (file extension) of the URL. +*/
+ long time;                     /*+ The time of the file (access or modification as appropriate). +*/
 }
 FileIndex;
 
@@ -55,7 +56,7 @@ typedef enum _SortMode
  ATime,                         /*+ Sort by access time. +*/
  Alpha,                         /*+ Sort Alphabetically. +*/
  Type,                          /*+ Sort by type (file extension). +*/
- Latest,                        /*+ Sort all files in one list. +*/
+ Latest,                        /*+ Sort all files in one list by modification time. +*/
  NSortModes                     /*+ The number of sort modes. +*/
 }
 SortMode;
@@ -68,13 +69,14 @@ int nfiles=0;
 
 static void IndexRoot(int fd,char *args,SortMode mode);
 static void IndexDir(int fd,char *dirname,SortMode mode);
+static void IndexOutgoing(int fd,SortMode mode);
 static void IndexLatest(int fd);
 
 static void CorrectIndex(int fd,char *path,char *args);
 static void IllegalIndex(int fd,char *path,char *args);
 
 static void add_dir(char *name,struct stat *buf,SortMode mode);
-static void add_file(char *name,struct stat *buf,SortMode mode,char *dir);
+static void add_file(char *name,struct stat *buf,SortMode mode,char *dir,char *outgoing);
 
 static int sort_alpha(FileIndex **a,FileIndex **b);
 static int sort_type(FileIndex **a,FileIndex **b);
@@ -122,7 +124,7 @@ void IndexPage(int fd,char *path,char *args)
        if(!strcmp(args,sorttype[sort][0]))
           break;
 
- if(strchr(newpath,'/') || !strcmp(newpath,"..") || !strcmp(newpath,".") || !strcmp(newpath,"outgoing") ||
+ if(strchr(newpath,'/') || !strcmp(newpath,"..") || !strcmp(newpath,".") ||
     (*newpath && (stat(newpath,&buf) || !S_ISDIR(buf.st_mode))))
     IllegalIndex(fd,newpath,args);
  else if(sort==NSortModes)
@@ -146,32 +148,16 @@ void IndexPage(int fd,char *path,char *args)
 
     if(!*newpath)
       {
-       write(fd,"</p>\n<ul>\n",10);
-
        if(sort==Latest)
           IndexLatest(fd);
        else
           IndexRoot(fd,args,sort);
-
-       write(fd,"</ul>\n",6);
-
-       if(sort==Latest)
-         {
-          sprintf(buffer,"<p align=center><a href=\"/index/?none\">[Back to the Hosts Index]</a></p>\n");
-          write(fd,buffer,strlen(buffer));
-         }
       }
+    else if(!strcmp(newpath,"outgoing"))
+       IndexOutgoing(fd,sort);
     else
-      {
-       sprintf(buffer,"</p>\n<b>http://%s/</b>\n<ul>\n",newpath);
-       write(fd,buffer,strlen(buffer));
-
        IndexDir(fd,newpath,sort);
 
-       sprintf(buffer,"</ul>\n<p align=center><a href=\"/index/?%s\">[Back to the Hosts Index]</a></p>\n",args);
-       write(fd,buffer,strlen(buffer));
-      }
-
     write(fd,tail,strlen(tail));
    }
 
@@ -195,6 +181,7 @@ static void IndexRoot(int fd,char *args,SortMode mode)
  DIR *dir;
  struct dirent* ent;
  int i;
+ char buffer[80];
 
  /* Open the spool directory. */
 
@@ -228,23 +215,35 @@ static void IndexRoot(int fd,char *args,SortMode mode)
  else if(mode==Alpha || mode==Type)
     qsort(files,nfiles,sizeof(FileIndex*),(int (*)(const void*,const void*))sort_alpha);
 
- for(i=0;i<nfiles;i++)
-   {
-    char *buffer=(char*)malloc(2*strlen(files[i]->name)+strlen(args)+40);
+ sprintf(buffer,"<p>\n<b>Server Hosts</b>\n");
+ write(fd,buffer,strlen(buffer));
 
-    sprintf(buffer,"<li><a href=\"/index/%s?%s\">%s</a>\n",files[i]->name,args,files[i]->name);
+ if(nfiles)
+   {
+    sprintf(buffer,"<p>\n%d Hosts\n",nfiles);
     write(fd,buffer,strlen(buffer));
 
-    free(buffer);
-    free(files[i]->name);
-    free(files[i]);
-   }
+    write(fd,"<ul>\n",5);
 
- if(nfiles==0)
-   {
-    char *msg="<li>No hosts.\n";
-    write(fd,msg,strlen(msg));
+    for(i=0;i<nfiles;i++)
+      {
+       char *buffer=(char*)malloc(2*strlen(files[i]->name)+strlen(args)+40);
+
+       sprintf(buffer,"<li><a href=\"/index/%s?%s\">%s</a>\n",files[i]->name,args,files[i]->name);
+       write(fd,buffer,strlen(buffer));
+
+       free(buffer);
+       free(files[i]->name);
+       free(files[i]);
+      }
+
+    write(fd,"</ul>\n",6);
    }
+ else
+    write(fd,"<p>\nNo hosts.\n",14);
+
+ sprintf(buffer,"<p align=center>\n<a href=\"/index/outgoing?%s\">[Outgoing&nbsp;Requests]</a>\n",sorttype[mode][0]);
+ write(fd,buffer,strlen(buffer));
 
  free(files);
  files=NULL;
@@ -267,6 +266,25 @@ static void IndexDir(int fd,char *dirname,SortMode mode)
  DIR *dir;
  struct dirent* ent;
  int i;
+ char *buffer;
+ char name[257],link[257];
+ struct stat buf;
+
+ /* Check if this is a link. */
+
+ strcpy(name,dirname);
+ link[0]=0;
+
+ while(!lstat(name,&buf) && S_ISLNK(buf.st_mode))
+   {
+    int n;
+
+    if((n=readlink(name,link,256))==-1)
+      {PrintMessage(Warning,"Cannot read link '%s' [%!s].",name);*link=0;break;}
+
+    link[n]=0;
+    strcpy(name,link);
+   }
 
  /* Open the spool subdirectory. */
 
@@ -291,7 +309,7 @@ static void IndexDir(int fd,char *dirname,SortMode mode)
     if(stat(ent->d_name,&buf))
        PrintMessage(Inform,"Cannot stat file '%s/%s' [%!s]; race condition?",dirname,ent->d_name);
     else if(S_ISREG(buf.st_mode))
-       add_file(ent->d_name,&buf,mode,NULL);
+       add_file(ent->d_name,&buf,mode,NULL,NULL);
    }
 
  closedir(dir);
@@ -307,45 +325,215 @@ static void IndexDir(int fd,char *dirname,SortMode mode)
  else if(mode==Type)
     qsort(files,nfiles,sizeof(FileIndex*),(int (*)(const void*,const void*))sort_type);
 
- for(i=0;i<nfiles;i++)
+ buffer=(char*)malloc(80+strlen(dirname));
+ sprintf(buffer,"<p>\n<b>http://%s/</b>\n",dirname);
+ write(fd,buffer,strlen(buffer));
+ free(buffer);
+
+ if(*link)
    {
-    char *buffer=(char*)malloc(4*strlen(files[i]->name)+3*strlen(dirname)+160);
-    char *url=(char*)malloc(strlen(files[i]->name)+strlen(dirname)+8),*encurl;
+    buffer=(char*)malloc(80+strlen(link));
+    sprintf(buffer,"=&gt; <i>http://%s/</i>\n",link);
+    write(fd,buffer,strlen(buffer));
+    free(buffer);
+   }
 
-    strcpy(url,dirname);
-    strcat(url,"/");
-    if(files[i]->name[0]=='/')
-       strcat(url,&files[i]->name[1]);
-    else
-       strcat(url,files[i]->name);
-    encurl=UrlEncode(url);
+ if(nfiles)
+   {
+    buffer=(char*)malloc(24);
+    sprintf(buffer,"<p>\n%d Pages\n",nfiles);
+    write(fd,buffer,strlen(buffer));
+    free(buffer);
 
-    if(strstr(files[i]->name,"?!"))
-       sprintf(buffer,"<li>[PostedNoRefresh]"
-                      "&nbsp;<a href=\"http://%s/%s\">%s</a>\n",
-               dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,
-               files[i]->name);
-    else
-       sprintf(buffer,"<li>[<a href=\"/refresh/%s/%s\">Refresh</a>|"
-                      "<a href=\"/refresh/?%s\">Options</a>]"
-                      "&nbsp;<a href=\"http://%s/%s\">%s</a>\n",
-               dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,
-               encurl,
-               dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,files[i]->name);
+    write(fd,"<ul>\n",5);
+
+    for(i=0;i<nfiles;i++)
+      {
+       char *url=(char*)malloc(strlen(files[i]->name)+strlen(dirname)+8),*encurl,*buffer;
+
+       strcpy(url,dirname);
+       strcat(url,"/");
+       if(files[i]->name[0]=='/')
+          strcat(url,&files[i]->name[1]);
+       else
+          strcat(url,files[i]->name);
+       encurl=UrlEncode(url);
+
+       buffer=(char*)malloc(3*strlen(files[i]->name)+2*strlen(dirname)+2*strlen(encurl)+256);
+       if(strstr(files[i]->name,"?!"))
+          sprintf(buffer,"<li>[Del"
+                         "|Refresh|"
+                         "Opt]"
+                         "&nbsp;<a href=\"http://%s/%s\">%s</a>\n",
+                  dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,
+                  files[i]->name);
+       else
+          sprintf(buffer,"<li>[<a href=\"/control/delete?url=%s\">Del</a>"
+                         "|<a href=\"/refresh/%s/%s\">Refresh</a>|"
+                         "<a href=\"/refresh/?%s\">Opt</a>]"
+                         "&nbsp;<a href=\"http://%s/%s\">%s</a>\n",
+                  encurl,
+                  dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,
+                  encurl,
+                  dirname,files[i]->name[0]=='/'?&files[i]->name[1]:files[i]->name,files[i]->name);
+       write(fd,buffer,strlen(buffer));
+
+       free(url);
+       free(encurl);
+       free(buffer);
+       free(files[i]->name);
+       free(files[i]);
+      }
+
+    write(fd,"</ul>\n",6);
+   }
+ else
+    write(fd,"<p>\nNo pages on this host.\n",27);
+
+ buffer=(char*)malloc(80);
+ sprintf(buffer,"<p align=center><a href=\"/index/?%s\">[Back to the Hosts Index]</a></p>\n",sorttype[mode][0]);
+ write(fd,buffer,strlen(buffer));
+ free(buffer);
+
+ free(files);
+ files=NULL;
+ nfiles=0;
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Create an index of the requests that are waiting in the outgoing directory.
+
+  int fd The file descriptor to write to.
+
+  SortMode mode The method to use to sort the names.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void IndexOutgoing(int fd,SortMode mode)
+{
+ DIR *dir;
+ struct dirent* ent;
+ int i;
+ char buffer[80];
+
+ /* Open the spool subdirectory. */
+
+ if(chdir("outgoing"))
+   {PrintMessage(Warning,"Cannot change to directory 'outgoing' [%!s]; not indexed.");return;}
+
+ dir=opendir(".");
+ if(!dir)
+   {PrintMessage(Warning,"Cannot open directory 'outgoing' [%!s]; not indexed.");chdir("..");return;}
+
+ ent=readdir(dir);  /* skip .  */
+ if(!ent)
+   {PrintMessage(Warning,"Cannot read directory 'outgoing' [%!s]; not indexed.");closedir(dir);chdir("..");return;}
+ ent=readdir(dir);  /* skip .. */
+
+ /* Print all of the file names. */
+
+ while((ent=readdir(dir)))
+   {
+    struct stat buf;
+
+    if(stat(ent->d_name,&buf))
+       PrintMessage(Inform,"Cannot stat file 'outgoing/%s' [%!s]; race condition?",ent->d_name);
+    else if(S_ISREG(buf.st_mode) && strncmp(ent->d_name,"tmp",3))
+      {
+       FILE *file=fopen(ent->d_name,"r");
+
+       if(file)
+         {
+          struct utimbuf utbuf;
+          char *url,*req;
+
+          req=ParseRequest(file,&url);
+          if(url)
+            {
+             char *host,*path,*args;
+
+             SplitURL(url,&host,&path,&args);
+             if(args)
+                sprintf(url,"%s/%s?%s",host,*path=='/'?&path[1]:path,args);
+             else
+                sprintf(url,"%s/%s",host,*path=='/'?&path[1]:path);
+
+             add_file(url,&buf,mode,NULL,ent->d_name);
+             free(url);
+            }
+
+          fclose(file);
+          if(req)
+             free(req);
+
+          utbuf.actime=buf.st_atime;
+          utbuf.modtime=buf.st_mtime;
+          utime(ent->d_name,&utbuf);
+         }
+      }
+   }
+
+ closedir(dir);
+
+ chdir("..");
+
+ /* Sort the files. */
+
+ if(mode==MTime || mode==ATime)
+    qsort(files,nfiles,sizeof(FileIndex*),(int (*)(const void*,const void*))sort_time);
+ else if(mode==Alpha)
+    qsort(files,nfiles,sizeof(FileIndex*),(int (*)(const void*,const void*))sort_alpha);
+ else if(mode==Type)
+    qsort(files,nfiles,sizeof(FileIndex*),(int (*)(const void*,const void*))sort_type);
+
+ sprintf(buffer,"<p>\n<b>Outgoing Requests</b>\n");
+ write(fd,buffer,strlen(buffer));
+
+ if(nfiles)
+   {
+    sprintf(buffer,"<p>\n%d Requests\n",nfiles);
     write(fd,buffer,strlen(buffer));
 
-    free(url);
-    free(encurl);
-    free(buffer);
-    free(files[i]->name);
-    free(files[i]);
-   }
+    write(fd,"<ul>\n",5);
 
- if(nfiles==0)
-   {
-    char *msg="<li>No files on this host.\n";
-    write(fd,msg,strlen(msg));
+    for(i=0;i<nfiles;i++)
+      {
+       char *buffer,*encurl;
+
+       encurl=UrlEncode(files[i]->name);
+
+       buffer=(char*)malloc(2*strlen(files[i]->name)+strlen(files[i]->file)+strlen(encurl)+256);
+       if(strstr(files[i]->name,"?!"))
+          sprintf(buffer,"<li>[<a href=\"/control/delete?req=%s\">Del</a>"
+                         "|Refresh|"
+                         "Opt]"
+                         "&nbsp;<a href=\"http://%s\">%s</a>\n",
+                  files[i]->file,
+                  files[i]->name,files[i]->name);
+       else
+          sprintf(buffer,"<li>[<a href=\"/control/delete?req=%s\">Del</a>"
+                         "|Refresh|"
+                         "<a href=\"/refresh/?%s\">Opt</a>]"
+                         "&nbsp;<a href=\"http://%s\">%s</a>\n",
+                  files[i]->file,
+                  encurl,
+                  files[i]->name,files[i]->name);
+       write(fd,buffer,strlen(buffer));
+
+       free(encurl);
+       free(buffer);
+       free(files[i]->name);
+       free(files[i]->file);
+       free(files[i]);
+      }
+
+    write(fd,"</ul>\n",6);
    }
+ else
+    write(fd,"<p>\nNo requests in outgoing directory.\n",39);
+
+ sprintf(buffer,"<p align=center><a href=\"/index/?%s\">[Back to the Hosts Index]</a></p>\n",sorttype[mode][0]);
+ write(fd,buffer,strlen(buffer));
 
  free(files);
  files=NULL;
@@ -365,7 +553,8 @@ static void IndexLatest(int fd)
  struct dirent* ent;
  int i;
  time_t now=time(NULL);
- int lastdays=0;
+ int lastdays=0,lasthours=0;
+ char buffer[80];
 
  /* Open the spool directory. */
 
@@ -412,7 +601,7 @@ static void IndexLatest(int fd)
              if(stat(ent2->d_name,&buf))
                 PrintMessage(Inform,"Cannot stat file '%s/%s' [%!s]; race condition?",ent->d_name,ent2->d_name);
              else if(S_ISREG(buf.st_mode) && (now-buf.st_mtime)<=IndexLatestDays*(24*3600))
-                add_file(ent2->d_name,&buf,Latest,ent->d_name);
+                add_file(ent2->d_name,&buf,Latest,ent->d_name,NULL);
             }
 
           closedir(dir2);
@@ -430,33 +619,43 @@ static void IndexLatest(int fd)
 
  qsort(files,nfiles,sizeof(FileIndex*),(int (*)(const void*,const void*))sort_time);
 
+ sprintf(buffer,"<p>\n<b>Latest Pages</b>\n<ul>\n");
+ write(fd,buffer,strlen(buffer));
+
  for(i=0;i<nfiles;i++)
    {
     char *encurl=UrlEncode(files[i]->name);
-    char *buffer=(char*)malloc(3*strlen(files[i]->name)+strlen(encurl)+160);
-    int days=(now-files[i]->time)/(24*3600);
+    char *buffer=(char*)malloc(3*strlen(files[i]->name)+2*strlen(encurl)+256);
+    int days=(now-files[i]->time)/(24*3600),hours=(now-files[i]->time)/3600;
 
     if(lastdays<days)
       {
-       char *msg="</ul>\n";
-       write(fd,msg,strlen(msg));
+       write(fd,"</ul>\n",6);
        for(;lastdays<days;lastdays++)
-         {
-          msg="<hr width=\"50%\">\n";
-          write(fd,msg,strlen(msg));
-         }
-       msg="<ul>\n";
-       write(fd,msg,strlen(msg));
+          write(fd,"<hr width=\"50%\">\n",17);
+       write(fd,"<ul>\n",5);
       }
+    else if(i && lasthours<hours)
+      {
+       write(fd,"</ul>\n",6);
+       write(fd,"<p>\n",4);
+       write(fd,"<ul>\n",5);
+      }
+    lasthours=hours;
 
     if(strstr(files[i]->name,"?!"))
-       sprintf(buffer,"<li>[PostedNoRefresh]"
+       sprintf(buffer,"<li>[<a href=\"/control/delete?url=%s\">Del</a>"
+                      "|Refresh|"
+                      "Opt]"
                       "&nbsp;<a href=\"http://%s\">%s</a>\n",
+               encurl,
                files[i]->name,files[i]->name);
     else
-       sprintf(buffer,"<li>[<a href=\"/refresh/%s\">Refresh</a>|"
-                      "<a href=\"/refresh/?%s\">Options</a>]"
+       sprintf(buffer,"<li>[<a href=\"/control/delete?url=%s\">Del</a>"
+                      "|<a href=\"/refresh/%s\">Refresh</a>|"
+                      "<a href=\"/refresh/?%s\">Opt</a>]"
                       "&nbsp;<a href=\"http://%s\">%s</a>\n",
+               encurl,
                files[i]->name,
                encurl,
                files[i]->name,files[i]->name);
@@ -470,17 +669,17 @@ static void IndexLatest(int fd)
 
  if(++lastdays<IndexLatestDays)
    {
-    char *msg="</ul>\n";
-    write(fd,msg,strlen(msg));
+    write(fd,"</ul>\n",6);
     for(;lastdays<IndexLatestDays;lastdays++)
-      {
-       msg="<hr width=\"50%\">\n";
-       write(fd,msg,strlen(msg));
-      }
-    msg="<ul>\n";
-    write(fd,msg,strlen(msg));
+       write(fd,"<hr width=\"50%\">\n",17);
+    write(fd,"<ul>\n",5);
    }
 
+ write(fd,"</ul>\n",6);
+
+ sprintf(buffer,"<p align=center><a href=\"/index/?%s\">[Back to the Hosts Index]</a></p>\n","none");
+ write(fd,buffer,strlen(buffer));
+
  free(files);
  files=NULL;
  nfiles=0;
@@ -635,34 +834,49 @@ static void add_dir(char *name,struct stat *buf,SortMode mode)
 /*++++++++++++++++++++++++++++++++++++++
   Add a file to the list.
 
-  char *name The name of the file.
+  char *name The name of the file or the URL if in outgoing.
 
   struct stat *buf The stat result on the file.
 
   SortMode mode The sort mode.
 
   char *dir The directory that the file is in for the 'Latest' mode.
+
+  char *outgoing The name of the file in the outgoing directory (if any).
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void add_file(char *name,struct stat *buf,SortMode mode,char *dir)
+static void add_file(char *name,struct stat *buf,SortMode mode,char *dir,char *outgoing)
 {
- int i;
- char *args="";
+ if(name[0] && name[0]==PATH_SEP &&
+    name[1] && name[1]==PATH_SEP)
+    return;
 
- for(i=0;name[i];i++)
-    if(name[i]==PATH_SEP)
-      {
-       if(name[i+1]!=PATH_SEP)
-          name[i]='/';
-       else
-          if(name[i+1] && name[i+2]!=PATH_SEP)
-            {args=&name[i];break;}
-          else
-            {name[i]='/';args=&name[i+1];break;}
-      }
-
- if(args!=name)
+ if(!(nfiles%16))
    {
+    if(!files)
+       files=(FileIndex**)malloc(16*sizeof(FileIndex*));
+    else
+       files=(FileIndex**)realloc(files,(nfiles+16)*sizeof(FileIndex*));
+   }
+ files[nfiles]=(FileIndex*)malloc(sizeof(FileIndex));
+
+ if(!outgoing)
+   {
+    char *args="";
+    int i;
+
+    for(i=0;name[i];i++)
+       if(name[i]==PATH_SEP)
+         {
+          if(name[i+1]!=PATH_SEP)
+             name[i]='/';
+          else
+             if(name[i+1] && name[i+2]!=PATH_SEP)
+               {args=&name[i];break;}
+             else
+               {name[i]='/';args=&name[i+1];break;}
+         }
+
     if(*args)
       {
        char *old_args=args;
@@ -692,20 +906,11 @@ static void add_file(char *name,struct stat *buf,SortMode mode,char *dir)
        *old_args=0;
       }
 
-    if(!(nfiles%16))
-      {
-       if(!files)
-          files=(FileIndex**)malloc(16*sizeof(FileIndex*));
-       else
-          files=(FileIndex**)realloc(files,(nfiles+16)*sizeof(FileIndex*));
-      }
-    files[nfiles]=(FileIndex*)malloc(sizeof(FileIndex));
-
     files[nfiles]->name=(char*)malloc((dir?strlen(dir)+2:0)+strlen(name)+strlen(args)+2);
     if(*args)
       {
        if(dir)
-          sprintf(files[nfiles]->name,"%s/%s?%s",dir,*name=='/'?"":name,args);
+          sprintf(files[nfiles]->name,"%s/%s?%s",dir,*name=='/'?name+1:name,args);
        else
           sprintf(files[nfiles]->name,"%s?%s",name,args);
 
@@ -714,27 +919,36 @@ static void add_file(char *name,struct stat *buf,SortMode mode,char *dir)
     else
       {
        if(dir)
-          sprintf(files[nfiles]->name,"%s/%s",dir,*name=='/'?"":name);
+          sprintf(files[nfiles]->name,"%s/%s",dir,*name=='/'?name+1:name);
        else
           strcpy(files[nfiles]->name,name);
       }
-
-    if(mode==Type)
-      {
-       files[nfiles]->type="";
-       for(i=strlen(name);i>0;i--)
-          if(files[nfiles]->name[i]=='.')
-             files[nfiles]->type=&files[nfiles]->name[i];
-          else if(files[nfiles]->name[i]=='/')
-             break;
-      }
-    else if(mode==MTime || mode==Latest)
-       files[nfiles]->time=buf->st_mtime;
-    else if(mode==ATime)
-       files[nfiles]->time=buf->st_atime;
-
-    nfiles++;
    }
+ else
+   {
+    files[nfiles]->name=(char*)malloc(strlen(name)+1);
+    strcpy(files[nfiles]->name,name);
+
+    files[nfiles]->file=(char*)malloc(strlen(outgoing)+1);
+    strcpy(files[nfiles]->file,outgoing);
+   }
+
+ if(mode==Type)
+   {
+    int i;
+    files[nfiles]->type="";
+    for(i=strlen(name);i>0;i--)
+       if(files[nfiles]->name[i]=='.')
+          files[nfiles]->type=&files[nfiles]->name[i];
+       else if(files[nfiles]->name[i]=='/')
+          break;
+   }
+ else if(mode==MTime || mode==Latest)
+    files[nfiles]->time=buf->st_mtime;
+ else if(mode==ATime)
+    files[nfiles]->time=buf->st_atime;
+
+ nfiles++;
 }
 
 
