@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/messages.c 1.18 1997/06/23 20:16:15 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/messages.c 1.23 1997/09/16 18:54:22 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3.
   Messages that the program sends in reply to requests from browsers
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -149,19 +149,23 @@ void CantRefreshPosted(int fd,char *url)
 
   char *url The url we are going to get.
 
+  char *outgoing Set to the name of the outgoing file.
+
   int already Set to true if we have already marked this for getting.
   ++++++++++++++++++++++++++++++++++++++*/
 
-void WillGetURL(int fd,char *url,int already)
+void WillGetURL(int fd,char *url,char *outgoing,int already)
 {
- char *head=
- "HTTP/1.0 404 WWWOFFLE Will Get\r\n" /* This line must not be changed (see wwwoffle.c). */
+ char *head1=
+ "HTTP/1.0 404 WWWOFFLE Will Get\r\n" /* This line must not be changed (see wwwoffle.c, spool.c). */
  "Content-type: text/html\r\n"
  "\r\n"
  "<HTML>\n"
  "<HEAD>\n"
  "<TITLE>\n"
- "WWWOFFLE - Will Get\n"
+ "WWWOFFLE - Will Get ";
+ char *head2=
+ "\n"
  "</TITLE>\n"
  "</HEAD>\n"
  "<BODY>\n";
@@ -181,27 +185,34 @@ void WillGetURL(int fd,char *url,int already)
  char *middle4b=
  "has already been recorded for download.\n";
  char *middle5=
- "<p align=center>[Refresh|<a href=\"http://%s:%d/refresh/?%s\">Options</a>]</p>\n";
+ "<p align=center>"
+ "[<a href=\"http://%s:%d/control/delete?req=%s\">Cancel</a>"
+ "|Refresh|"
+ "<a href=\"http://%s:%d/refresh/?%s\">Options</a>]</p>\n";
  char *tail=
  "</BODY>\n"
  "</HTML>\n";
  char *encurl=UrlEncode(url);
  char *localhost=GetLocalHost();
- char *buffer=(char*)malloc(strlen(middle5)+strlen(localhost)+strlen(encurl)+8);
+ char *buffer=(char*)malloc(strlen(middle5)+2*strlen(localhost)+strlen(encurl)+strlen(outgoing)+16);
 
- write(fd,head,strlen(head));
- if(already)
-    write(fd,middle1b,strlen(middle1b));
- else
+ write(fd,head1,strlen(head1));
+ write(fd,url,strlen(url));
+ write(fd,head2,strlen(head2));
+ if(!already)
     write(fd,middle1a,strlen(middle1a));
+ else
+    write(fd,middle1b,strlen(middle1b));
  write(fd,middle2,strlen(middle2));
  write(fd,url,strlen(url));
  write(fd,middle3,strlen(middle3));
- if(already)
-    write(fd,middle4b,strlen(middle4b));
- else
+ if(!already)
     write(fd,middle4a,strlen(middle4a));
- sprintf(buffer,middle5,localhost,HTTP_Port,encurl);
+ else
+    write(fd,middle4b,strlen(middle4b));
+ sprintf(buffer,middle5,
+         localhost,HTTP_Port,outgoing,
+         localhost,HTTP_Port,encurl);
  write(fd,buffer,strlen(buffer));
  write(fd,tail,strlen(tail));
 
@@ -278,7 +289,7 @@ void WelcomePage(int fd,int password)
  "<HTML>\n"
  "<HEAD>\n"
  "<TITLE>\n"
- "WWWOFFLE - Proxy Welcome Page\n"
+ "WWWOFFLE Version 1.3 - Proxy Welcome Page\n"
  "</TITLE>\n"
  "</HEAD>\n"
  "<BODY>\n"
@@ -306,6 +317,7 @@ void WelcomePage(int fd,int password)
  "<ul>\n"
  "<li> Downloading of specified pages non-interactively.\n"
  "<li> Can automatically fetch inlined images in pages fetched this way.\n"
+ "<li> Can automatically fetch contents of all frames on pages fetched this way.\n"
  "<li> Automatically follows links for pages that have been moved.\n"
  "</ul>\n"
  "Provides\n"
@@ -315,6 +327,7 @@ void WelcomePage(int fd,int password)
  "<li> Interactive or command line control of online/offline status.\n"
  "<li> User selectable purging of pages from cache based on hostname.\n"
  "<li> Interactive or command line option to fetch pages and links recursively.\n"
+ "<li> Interactive web page to allow editing of the configuration file.\n"
  "</ul>\n"
  "General\n"
  "<ul>\n"
@@ -341,6 +354,8 @@ void WelcomePage(int fd,int password)
  "<dd>To have the indexes sorted alphabetically according to the file name.\n"
  "<dt><a href=\"/index/?type\">/index/?type</a>\n"
  "<dd>To have the indexes sorted alphabetically according to the file extension.\n"
+ "<dt><a href=\"/index/outgoing?none\">/index/outgoing?none</a>\n"
+ "<dd>The list of requests waiting to be fetched in the order they were requested.\n"
  "<dt><a href=\"/index/?latest\">/index/?latest</a>\n"
  "<dd>A flat index of all files modified in the few days sorted by modification time.\n"
  "</dl>\n"
@@ -365,8 +380,8 @@ void WelcomePage(int fd,int password)
  "<a href=\"http://www.gedanken.demon.co.uk/wwwoffle/index.html\">http://www.gedanken.demon.co.uk/wwwoffle/index.html</a>\n"
  "and contains the latest information about the program in general.\n"
  "<p>\n"
- "The latest information about using this version of WWWOFFLE is on the WWWOFFLE Version 1.2 Users Page at\n"
- "<a href=\"http://www.gedanken.demon.co.uk/wwwoffle/version-1.2/user.html\">http://www.gedanken.demon.co.uk/wwwoffle/version-1.2/user.html</a>\n"
+ "The latest information about using this version of WWWOFFLE is on the WWWOFFLE Version 1.3 Users Page at\n"
+ "<a href=\"http://www.gedanken.demon.co.uk/wwwoffle/version-1.3/user.html\">http://www.gedanken.demon.co.uk/wwwoffle/version-1.3/user.html</a>\n"
  "and contains more information about using this version of the program.\n"
  "<p>\n"
  "You can e-mail the author at <a href=\"mailto:amb@gedanken.demon.co.uk\">amb@gedanken.demon.co.uk</a>,\n"
@@ -426,3 +441,19 @@ void IllegalPage(int fd,char *path,char *args)
  write(fd,string,strlen(string));
  write(fd,tail,strlen(tail));
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Return a page stating that the requested page has not been modified.
+
+  int fd The file descriptor to write to.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+void NotModified(int fd)
+{
+ char *page=
+ "HTTP/1.0 304 WWWOFFLE Not Modified\r\n"
+ "\r\n";
+
+ write(fd,page,strlen(page));
+}
