@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/parse.c 1.23 1997/06/26 16:07:31 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/parse.c 1.29 1997/09/23 18:08:51 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3.
   Functions to parse the HTTP requests.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -68,10 +68,7 @@ char *ParseRequest(FILE *req,char **url)
     if(!strncmp("Referer:",buf,8) && (pling=strstr(buf,"?!")))
        strcpy(pling,"\r\n");
 
-    if(!strncmp("If-Modified-Since:",buf,18))
-       continue;
-
-    if(!strncmp("Content-length:",buf,15))
+    if(!strncmp("Content-length:",buf,15) || !strncmp("Content-Length:",buf,15))
        length=atoi(&buf[15]);
 
     if(n)
@@ -403,6 +400,8 @@ void SplitURL(char *url,char **host,char **path,char **args)
 
  if(!*path || !**path)
     *path="";
+ else
+    *path=UrlDecode(*path,0);
 
  for(i=0;(*host)[i] && (*host)[i]!=':';i++)
     (*host)[i]=tolower((*host)[i]);
@@ -481,21 +480,76 @@ char *RFC822Date(long t,int utc)
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Convert a string representing a date into a time.
+
+  long DateToTimeT Returns the time.
+
+  char *date The date string.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+long DateToTimeT(char *date)
+{
+ static char *month[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};
+ int  year,day,hour,min,sec;
+ char monthstr[16];
+ long retval=0;
+
+ if(sscanf(date,"%*s %d %s %d %d:%d:%d",&day,monthstr,&year,&hour,&min,&sec)==6 ||
+    sscanf(date,"%*s %d-%3s-%d %d:%d:%d",&day,monthstr,&year,&hour,&min,&sec)==6 ||
+    sscanf(date,"%*s %3s %d %d:%d:%d %d",monthstr,&day,&hour,&min,&sec,&year)==6)
+   {
+    struct tm tim;
+    int mon;
+
+    for(mon=0;mon<12;mon++)
+       if(!strcmp(monthstr,month[mon]))
+          break;
+
+    tim.tm_sec=sec;
+    tim.tm_min=min;
+    tim.tm_hour=hour;
+    tim.tm_mday=day;
+    tim.tm_mon=mon;
+    if(year<38)
+       tim.tm_year=year+100;
+    else if(year<100)
+       tim.tm_year=year;
+    else
+       tim.tm_year=year-1900;
+    tim.tm_isdst=0;
+
+    retval=mktime(&tim);
+
+    if(retval==-1)
+       retval=0;
+   }
+ else if(sscanf(date,"%ld %1s",&retval,monthstr)==1)
+    ;
+
+ return(retval);
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   Decode a string that has been UrlEncoded.
 
   char *UrlDecode Returns a malloced copy of the decoded string.
 
   char *str The string to be decoded.
+
+  int isform Set to true if the string being decoded is from a form.
   ++++++++++++++++++++++++++++++++++++++*/
 
-char *UrlDecode(char *str)
+char *UrlDecode(char *str, int isform)
 {
  int i,j;
  char *copy=(char*)malloc(strlen(str)+1);
 
  for(i=0,j=0;str[i];i++)
-    if(str[i]=='%')
+    if(str[i]=='+' && isform)
+       copy[j++]=' ';
+    else if(str[i]=='%')
       {
        unsigned int val=0;
        i++;
@@ -532,10 +586,11 @@ char *UrlEncode(char *str)
  char *copy=(char*)malloc(3*strlen(str)+1);
 
  for(i=0,j=0;str[i];i++)
-    if((str[i]>='A' && str[i]<='Z') ||
-       (str[i]>='a' && str[i]<='z') ||
-       (str[i]>='0' && str[i]<='9') ||
-       str[i]=='/' || str[i]==':' || str[i]=='.')
+    if(isalpha(str[i]) ||
+       isdigit(str[i]) ||
+       str[i]=='/' || str[i]=='?' || str[i]==':' || str[i]=='@' || str[i]=='=' || str[i]=='&' ||
+       str[i]=='$' || str[i]=='-' || str[i]=='_' || str[i]=='.' || str[i]=='+' || str[i]=='!' ||
+       str[i]=='*' || str[i]=='(' || str[i]==')' || str[i]==',' || str[i]=='\'')
        copy[j++]=str[i];
     else
       {
@@ -557,3 +612,66 @@ char *UrlEncode(char *str)
 
  return(copy);
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Make the input string safe to output as HTML ( not < > & " ).
+
+  char* HTMLString Returns a safe HTML string.
+
+  char* c A non-safe HTML string.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+char* HTMLString(char* c)
+{
+ int i=0,j=0,len=256-5;              /* 5 is the longest possible inserted amount */
+ char* ret=(char*)malloc(257);
+
+ do
+   {
+    for(;j<len && c[i];i++)
+       switch(c[i])
+         {
+         case '<':
+          ret[j++]='&';
+          ret[j++]='l';
+          ret[j++]='t';
+          ret[j++]=';';
+          break;
+         case '>':
+          ret[j++]='&';
+          ret[j++]='g';
+          ret[j++]='t';
+          ret[j++]=';';
+          break;
+         case '"':
+          ret[j++]='&';
+          ret[j++]='q';
+          ret[j++]='u';
+          ret[j++]='o';
+          ret[j++]='t';
+          ret[j++]=';';
+          break;
+         case '&':
+          ret[j++]='&';
+          ret[j++]='a';
+          ret[j++]='m';
+          ret[j++]='p';
+          ret[j++]=';';
+          break;
+         default:
+          ret[j++]=c[i];
+         }
+
+    if(c[i])                 /* Not finished */
+      {
+       ret=(char*)realloc((void*)ret,len+256+5);
+       len+=256;
+      }
+   }
+ while(c[i]);
+
+ ret[j]=0;
+
+ return(ret);
+}
