@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/purge.c 1.11 1997/06/13 19:37:09 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/purge.c 1.13 1997/09/06 19:57:04 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2a.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3.
   Purge old files from the cache.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -16,6 +16,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <time.h>
 #include <utime.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -85,52 +86,81 @@ void PurgeCache(int fd)
 
        if(lstat(ent->d_name,&buf))
           PrintMessage(Inform,"Cannot stat file '%s' [%!s]; race condition?",ent->d_name);
-       else
-          if(S_ISDIR(buf.st_mode) && strcmp(ent->d_name,"outgoing"))
-            {
-             int bytes=0;
-             int age=WhatPurgeAge(ent->d_name);
-             char *msg=(char*)malloc(strlen(ent->d_name)+80);
+       else if(S_ISDIR(buf.st_mode) && strcmp(ent->d_name,"outgoing"))
+         {
+          int bytes=0;
+          int age=WhatPurgeAge(ent->d_name);
+          char *msg=(char*)malloc(strlen(ent->d_name)+80);
 
-             if(age<0)
-                bytes=PurgeFiles(ent->d_name,now,-1,nbytes_by_age);
+          if(age<0)
+             bytes=PurgeFiles(ent->d_name,now,-1,nbytes_by_age);
+          else
+            {
+             if(pass==2)
+                age=age_for_size;
+             bytes=PurgeFiles(ent->d_name,now,age,nbytes_by_age);
+
+             if(bytes==-1)
+               {
+#if DO_DELETE
+                if(rmdir(ent->d_name))
+                   PrintMessage(Warning,"Cannot delete what should be an empty directory '%s' [%!s].",ent->d_name);
+#endif
+               }
              else
                {
-                if(pass==2)
-                   age=age_for_size;
-                bytes=PurgeFiles(ent->d_name,now,age,nbytes_by_age);
+                struct utimbuf utbuf;
 
-                if(bytes==-1)
-                  {
-#if DO_DELETE
-                   if(rmdir(ent->d_name))
-                      PrintMessage(Warning,"Cannot delete 'empty' directory '%s' [%!s].",ent->d_name);
-#endif
-                  }
-                else
-                  {
-                   struct utimbuf utbuf;
-
-                   utbuf.actime=buf.st_atime;
-                   utbuf.modtime=buf.st_mtime;
-                   utime(ent->d_name,&utbuf);
-                  }
+                utbuf.actime=buf.st_atime;
+                utbuf.modtime=buf.st_mtime;
+                utime(ent->d_name,&utbuf);
                }
-
-             if(bytes>0)
-                total_bytes+=bytes;
-             if(bytes>=0)
-                total_dirs++;
-
-             if(age<0)
-                sprintf(msg,"Not Purged       %30s ; %5d kB.\n",ent->d_name,(bytes+512)/1024);
-             else if(bytes==-1)
-                sprintf(msg,"Purged (%2d days) %30s ; empty.\n",age,ent->d_name);
-             else
-                sprintf(msg,"Purged (%2d days) %30s ; %5d kB.\n",age,ent->d_name,(bytes+512)/1024);
-             write(fd,msg,strlen(msg));
-             free(msg);
             }
+
+          if(bytes>0)
+             total_bytes+=bytes;
+          if(bytes>=0)
+             total_dirs++;
+
+          if(age<0)
+             sprintf(msg,"Not Purged       %40s ; %5d kB.\n",ent->d_name,(bytes+512)/1024);
+          else if(bytes==-1)
+             sprintf(msg,"Purged (%2d days) %40s ; (empty) - deleted.\n",age,ent->d_name);
+          else
+             sprintf(msg,"Purged (%2d days) %40s ; %5d kB.\n",age,ent->d_name,(bytes+512)/1024);
+          write(fd,msg,strlen(msg));
+          free(msg);
+         }
+       else if(S_ISLNK(buf.st_mode))
+         {
+          char name[257],link[257],*msg;
+          struct stat buf2;
+
+          strcpy(name,ent->d_name);
+
+          while(!lstat(name,&buf2) && S_ISLNK(buf2.st_mode))
+            {
+             int n;
+
+             if((n=readlink(name,link,256))==-1)
+               {PrintMessage(Warning,"Cannot read link '%s' [%!s].",name);*link=0;break;}
+
+             link[n]=0;
+             strcpy(name,link);
+            }
+
+          msg=(char*)malloc(strlen(ent->d_name)+strlen(link)+80);
+
+          if(!*link)
+             sprintf(msg,"Mirror Link      %40s ; => ? (malformed).\n",ent->d_name);
+          else if(stat(link,&buf2))
+             sprintf(msg,"Mirror Link      %40s ; => %s (missing).\n",ent->d_name,link);
+          else
+             sprintf(msg,"Mirror Link      %40s ; => %s.\n",ent->d_name,link);
+
+          write(fd,msg,strlen(msg));
+          free(msg);
+         }
       }
 
     closedir(dir);
