@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/refresh.c 1.7 1997/06/26 16:08:00 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/refresh.c 1.14 1997/09/23 18:08:51 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3.
   The HTML interactive page to refresh a URL.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -26,11 +26,11 @@
 
 
 static void RefreshFormPage(int fd,char *args);
-static void ParseFormRequest(int fd,char *request,int is_online);
+static int  ParseFormRequest(int fd,char *request,int is_online);
 static void RefreshFormError(int fd,char *body);
-static void RefreshFormRecursive(int fd,char *url,char *method);
+static int  RefreshFormRecursive(int fd,char *url,char *method);
 static void IllegalRefreshPage(int fd,char *path);
-static void RecurseFetchPages(int fd,char *url,char *method);
+static int  RecurseFetchPages(int fd,char *url,char *method);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -58,13 +58,16 @@ char *RefreshPage(int fd,char *path,char *args,char *url,char **request,int is_o
  if(!strcmp("refresh/",path))
     RefreshFormPage(fd,args);
  else if(!strcmp("refresh-request/",path))
-    ParseFormRequest(fd,*request,is_online);
+   {
+    if(ParseFormRequest(fd,*request,is_online))
+       newurl=(char*)1;
+   }
  else if(!strncmp("refresh/",path,8))
    {
     char *copy,*u,*p;
 
     copy=(char*)malloc(strlen(*request)+1);
-    p=strstr(*request,path)+8;
+    p=strstr(*request,"refresh")+8;
     u=strstr(*request,url); *u=0;
     strcpy(copy,*request);
     strcat(copy,"http://");
@@ -73,7 +76,7 @@ char *RefreshPage(int fd,char *path,char *args,char *url,char **request,int is_o
     *request=copy;
 
     copy=(char*)malloc(strlen(url)+1);
-    p=strstr(url,path)+8;
+    p=strstr(url,"refresh")+8;
     strcpy(copy,"http://");
     strcat(copy,p);
     free(url);
@@ -93,7 +96,8 @@ char *RefreshPage(int fd,char *path,char *args,char *url,char **request,int is_o
           else
              strcpy(url,slash+1);
           *slash=0;
-          RecurseFetchPages(fd,url,path);
+          if(RecurseFetchPages(fd,url,path))
+             newurl=(char*)1;
           free(url);
          }
        else
@@ -146,19 +150,19 @@ static void RefreshFormPage(int fd,char *args)
  "<p>\n"
  "<form action=\"/refresh-request/\" method=post>\n"
  "Fetch <select name=\"method\">\n"
- "<option value=\"refresh\" selected>this URL only\n"
- "<option value=\"refresh-dir-1\"   >recursively to depth 1 in the same directory\n"
- "<option value=\"refresh-dir-2\"   >recursively to depth 2 in the same directory\n"
- "<option value=\"refresh-dir-3\"   >recursively to depth 3 in the same directory\n"
- "<option value=\"refresh-dir-4\"   >recursively to depth 4 in the same directory\n"
- "<option value=\"refresh-dir-5\"   >recursively to depth 5 in the same directory\n"
- "<option value=\"refresh-host-1\"  >recursively to depth 1 on the same host\n"
- "<option value=\"refresh-host-2\"  >recursively to depth 2 on the same host\n"
- "<option value=\"refresh-host-3\"  >recursively to depth 3 on the same host\n"
- "<option value=\"refresh-host-4\"  >recursively to depth 4 on the same host\n"
- "<option value=\"refresh-any-1\"   >recursively to depth 1 on any host\n"
- "<option value=\"refresh-any-2\"   >recursively to depth 2 on any host\n"
- "<option value=\"refresh-any-3\"   >recursively to depth 3 on any host\n"
+ "<option value=\"-none\" selected>this URL only\n"
+ "<option value=\"-dir-1\"   >recursively to depth 1 in the same directory\n"
+ "<option value=\"-dir-2\"   >recursively to depth 2 in the same directory\n"
+ "<option value=\"-dir-3\"   >recursively to depth 3 in the same directory\n"
+ "<option value=\"-dir-4\"   >recursively to depth 4 in the same directory\n"
+ "<option value=\"-dir-5\"   >recursively to depth 5 in the same directory\n"
+ "<option value=\"-host-1\"  >recursively to depth 1 on the same host\n"
+ "<option value=\"-host-2\"  >recursively to depth 2 on the same host\n"
+ "<option value=\"-host-3\"  >recursively to depth 3 on the same host\n"
+ "<option value=\"-host-4\"  >recursively to depth 4 on the same host\n"
+ "<option value=\"-any-1\"   >recursively to depth 1 on any host\n"
+ "<option value=\"-any-2\"   >recursively to depth 2 on any host\n"
+ "<option value=\"-any-3\"   >recursively to depth 3 on any host\n"
  "</select>\n"
  "<br>\n"
  "<input type=\"text\" name=\"url\" value=\"";
@@ -166,9 +170,13 @@ static void RefreshFormPage(int fd,char *args)
  "\"size=60>\n"
  "<br>\n"
  "<input name=\"images\" type=\"checkbox\" value=\"-images\" ";
- char *tail=
+ char *middle2=
  ">Fetch images in the pages\n"
  "<br>\n"
+ "<input name=\"frames\" type=\"checkbox\" value=\"-frames\" ";
+ char *tail=
+ ">Fetch frames in the pages\n"
+ "<br>\n"
  "<input name=\"force\" type=\"checkbox\" value=\"-force\">Force refresh even if already cached"
  "<br>\n"
  "<input type=\"submit\" value=\"Fetch Now\">\n"
@@ -178,14 +186,20 @@ static void RefreshFormPage(int fd,char *args)
 
  write(fd,head,strlen(head));
  if(args)
+    write(fd,args,strlen(args));
+/*
    {
-    char *copy=UrlDecode(args);
+    char *copy=UrlDecode(args,0);
     write(fd,copy,strlen(copy));
     free(copy);
    }
+*/
  write(fd,middle1,strlen(middle1));
  if(FetchImages)
     write(fd,"checked",7);
+ write(fd,middle2,strlen(middle2));
+ if(FetchFrames)
+    write(fd,"checked",7);
  write(fd,tail,strlen(tail));
 }
 
@@ -193,6 +207,8 @@ static void RefreshFormPage(int fd,char *args)
 /*++++++++++++++++++++++++++++++++++++++
   Parse the reply from the form.
 
+  int ParseFormRequest Returns a true value if there are more pages to get.
+
   int fd The file descriptor of the client.
 
   char *request The HTTP request sent by the browser.
@@ -200,10 +216,11 @@ static void RefreshFormPage(int fd,char *args)
   int is_online Set to true if we are online.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void ParseFormRequest(int fd,char *request,int is_online)
+static int ParseFormRequest(int fd,char *request,int is_online)
 {
  int i;
- char *body,*copy,*url=NULL,*method=NULL,*images="",*force="";
+ int more=0;
+ char *body,*copy,*url=NULL,*method=NULL,*total_method,*images="",*frames="",*force="";
 
  if((body=strstr(request,"\n\n")) ||
     (body=strstr(request,"\r\n\n")) ||
@@ -218,7 +235,7 @@ static void ParseFormRequest(int fd,char *request,int is_online)
  else
    {
     RefreshFormError(fd,NULL);
-    return;
+    return(0);
    }
 
  copy=(char*)malloc(strlen(body)+1);
@@ -234,6 +251,8 @@ static void ParseFormRequest(int fd,char *request,int is_online)
           method=&copy[i+7];
        if(!strncmp("images=",&copy[i],7))
           images=&copy[i+7];
+       if(!strncmp("frames=",&copy[i],7))
+          frames=&copy[i+7];
        if(!strncmp("force=",&copy[i],6))
           force=&copy[i+6];
        if(!strncmp("url=",&copy[i],4))
@@ -241,27 +260,34 @@ static void ParseFormRequest(int fd,char *request,int is_online)
       }
    }
 
- if(url==NULL || *url==0 || method==NULL || strchr(url,'+'))
+ if(url==NULL || *url==0 || method==NULL)
    {
     RefreshFormError(fd,body);
     free(copy);
-    return;
+    return(0);
    }
 
- url=UrlDecode(url);
+ url=UrlDecode(url,1);
 
  if(!strncasecmp(url,"http://",7))
     url+=7;
 
+ total_method=(char*)malloc(strlen(body));
+ strcpy(total_method,"refresh");
+ strcat(total_method,method);
+ strcat(total_method,images);
+ strcat(total_method,frames);
+ strcat(total_method,force);
+
  if(is_online)
-    RefreshFormRecursive(fd,url,method);
+    more=RefreshFormRecursive(fd,url,total_method);
  else
    {
-    char *new_request,*new_url;
+    char *new_request,*new_url,*outgoing_filename=NULL;
     int new_outgoing;
 
     new_url=(char*)malloc(strlen(body)+16);
-    sprintf(new_url,"/%s%s%s/%s",method,images,force,url);
+    sprintf(new_url,"/%s/%s",total_method,url);
     new_request=RequestURL(new_url,NULL);
     new_outgoing=OpenOutgoingSpoolFile(0);
 
@@ -270,15 +296,18 @@ static void ParseFormRequest(int fd,char *request,int is_online)
     else
       {
        write(new_outgoing,new_request,strlen(new_request));
-       CloseOutgoingSpoolFile(new_outgoing);
+       outgoing_filename=CloseOutgoingSpoolFile(new_outgoing);
+
+       WillGetURL(fd,url,outgoing_filename,0);
       }
 
     free(new_request);
-
-    WillGetURL(fd,url,0);
    }
 
+ free(total_method);
  free(copy);
+
+ return(more);
 }
 
 
@@ -334,6 +363,8 @@ static void RefreshFormError(int fd,char *body)
 /*++++++++++++++++++++++++++++++++++++++
   Fetch pages recursively, and show progress.
 
+  int RefreshFormRecursive Returns a true value if there are more pages to get.
+
   int fd The file descriptor to write into.
 
   char *url The URL to fetch.
@@ -341,8 +372,9 @@ static void RefreshFormError(int fd,char *body)
   char *method The method to use.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void RefreshFormRecursive(int fd,char *url,char *method)
+static int RefreshFormRecursive(int fd,char *url,char *method)
 {
+ int more;
  char *head=
  "HTTP/1.0 200 WWWOFFLE Refresh Recursive Page\r\n"
  "Content-type: text/html\r\n"
@@ -360,7 +392,7 @@ static void RefreshFormRecursive(int fd,char *url,char *method)
  char *middle=
  "\n"
  "</tt></b><br>\n"
- "and the links to the specified depth are being fetched.\n"
+ "and the links to the specified depth are being fetched in the background.\n"
  "<pre>\n";
  char *tail=
  "</pre>\n"
@@ -372,9 +404,11 @@ static void RefreshFormRecursive(int fd,char *url,char *method)
  write(fd,url,strlen(url));
  write(fd,middle,strlen(middle));
 
- RecurseFetchPages(fd,url,method);
+ more=RecurseFetchPages(fd,url,method);
 
  write(fd,tail,strlen(tail));
+
+ return(more);
 }
 
 
@@ -422,11 +456,11 @@ static void IllegalRefreshPage(int fd,char *path)
 }
 
 
-static void add_page(char ***pages,int *npages,char *page);
-
 /*++++++++++++++++++++++++++++++++++++++
   Fetch pages recursively.
 
+  int RecurseFetchPages Returns a true value if there are more pages to get.
+
   int fd The file descriptor to output to (mode==Real).
 
   char *url The url to start at.
@@ -434,14 +468,21 @@ static void add_page(char ***pages,int *npages,char *page);
   char *method The method to use, encoding the depth and other options.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void RecurseFetchPages(int fd,char *url,char *method)
+static int RecurseFetchPages(int fd,char *url,char *method)
 {
- int i;
- int recursive_depth=0,recursive_mode=0,recursive_images=0,force_fetch=0;
- int npages=0,npages_recursive=1;
- char **pages=NULL;
- char *buffer=(char*)malloc(257);
- char *dash,*copy;
+ int recursive_depth=0,recursive_mode=0,images=0,frames=0,force=0;
+ int more=0,status;
+ char *buffer;
+ char *dash;
+ int socket;
+ FILE *socket_f;
+ int parsed=0;
+ char *page_host,*page_path,*args;
+ char *request,*line;
+ char **list;
+ int j;
+
+ PrintMessage(Debug,"Refresh method='%s'",method);
 
  if(*method=='-')
     method++;
@@ -455,171 +496,284 @@ static void RecurseFetchPages(int fd,char *url,char *method)
        ;
     else if(!strcmp(method,"none"))
        ;
-    else if(!strcmp(method,"any"))
+    else if(!strcmp(method,"dir"))
        recursive_mode=1;
     else if(!strcmp(method,"host"))
-       recursive_mode=0;
-    else if(!strcmp(method,"dir"))
-       recursive_mode=-1;
+       recursive_mode=2;
+    else if(!strcmp(method,"any"))
+       recursive_mode=3;
     else if(!strcmp(method,"images"))
-       recursive_images=1;
+       images=1;
+    else if(!strcmp(method,"frames"))
+       frames=1;
     else if(!strcmp(method,"force"))
-       force_fetch=1;
+       force=1;
     else if(atoi(method))
        recursive_depth=atoi(method);
-    else
-       return;
 
     method=dash+1;
    }
  while(dash);
 
- copy=(char*)malloc(strlen(url)+8);
- sprintf(copy,"http://%s",url);
- add_page(&pages,&npages,copy);
- free(copy);
+ buffer=(char*)malloc(strlen(url)+8);
+ strcpy(buffer,"http://");
+ strcat(buffer,url);
+ url=buffer;
 
- for(i=0;i<npages;i++)
+ /* Get the page */
+
+ socket=OpenClientSocket("localhost",HTTP_Port);
+ if(socket==-1)
    {
-    int socket;
-    FILE *socket_f;
-    int parsed=0;
-    char *page_host,*page_path,*args;
-    char *request,*msg;
-    char **list;
-    int j;
+    PrintMessage(Warning,"Cannot open connection to wwwoffle proxy.");
+    return(0);
+   }
 
-    if(recursive_depth && i==npages_recursive)
-      {
-       recursive_depth--;
-       npages_recursive=npages;
-      }
+ socket_f=fdopen(socket,"r");
+ if(!socket_f)
+   {
+    PrintMessage(Warning,"Cannot fdopen the server connection [%!s].");
+    close(socket);
+    return(0);
+   }
 
-    socket=OpenClientSocket("localhost",HTTP_Port);
-    if(socket==-1)
-      {
-       PrintMessage(Warning,"Cannot open connection to wwwoffle proxy.");
-       continue;
-      }
-
-    socket_f=fdopen(socket,"r");
-    if(!socket_f)
-      {
-       PrintMessage(Warning,"Cannot fdopen the server connection [%!s].");
-       close(socket);
-       continue;
-      }
-
-    msg=(char*)malloc(strlen(pages[i])+16);
-    sprintf(msg,"Getting %s\n",pages[i]);
+ if(fd>=0)
+   {
+    char *msg=(char*)malloc(strlen(url)+16);
+    sprintf(msg,"Getting %s\n",url);
     write(fd,msg,strlen(msg));
     free(msg);
+   }
 
-    request=RequestURL(pages[i],NULL);
+ request=RequestURL(url,NULL);
 
-    if(force_fetch)
+ if(force)
+   {
+    char *copy=(char*)malloc(strlen(request)+24);
+    char *eol=strchr(request,'\n');
+
+    *eol=0;eol++;
+    strcpy(copy,request);
+    strcat(copy,"\nPragma: no-cache\r\n");
+    strcat(copy,eol);
+
+    free(request);
+    request=copy;
+   }
+
+ write(socket,request,strlen(request));
+
+ SplitURL(url,&page_host,&page_path,&args);
+
+ line=fgets_realloc(NULL,socket_f);
+ if(sscanf(line,"%*s %d",&status)!=1)
+    status=404;
+
+ parsed=ParseHTML(socket_f,page_host,page_path);
+
+ buffer=(char*)malloc(257);
+
+ while(fread(buffer,1,256,socket_f)>0);
+
+ fclose(socket_f);
+
+ free(line);
+ free(request);
+ free(buffer);
+
+ if(images && parsed && (list=ListImages()))
+    for(j=0;list[j];j++)
       {
-       char *copy=(char*)malloc(strlen(request)+24);
-       char *eol=strchr(request,'\n');
+       char *new_request=RequestURL(list[j],url);
+       int new_outgoing=OpenOutgoingSpoolFile(0);
 
-       *eol=0;eol++;
-       strcpy(copy,request);
-       strcat(copy,"\nPragma: no-cache\r\n");
-       strcat(copy,eol);
+       PrintMessage(Debug,"Image=%s",list[j]);
 
-       free(request);
-       request=copy;
-      }
-
-    write(socket,request,strlen(request));
-
-    SplitURL(pages[i],&page_host,&page_path,&args);
-
-    parsed=ParseHTML(socket_f,page_host,page_path);
-
-    while(fread(buffer,1,256,socket_f)>0);
-
-    if(recursive_images && parsed && (list=ListImages()))
-       for(j=0;list[j];j++)
+       if(new_outgoing==-1)
+          PrintMessage(Warning,"Cannot open the new outgoing request to write.");
+       else
          {
-          PrintMessage(Debug,"Image=%s",list[j]);
-
-          add_page(&pages,&npages,list[j]);
+          write(new_outgoing,new_request,strlen(new_request));
+          CloseOutgoingSpoolFile(new_outgoing);
+          more=1;
          }
 
-    if(recursive_depth && parsed && (list=ListLinks()))
+       free(new_request);
+      }
+
+ if(frames && parsed && (list=ListFrames()))
+    for(j=0;list[j];j++)
       {
-       for(j=0;list[j];j++)
+       char *new_request,*refresh;
+       int recurse=1;
+       int new_outgoing;
+       char *link_host=NULL,*link_path,*args;
+
+       SplitURL(list[j],&link_host,&link_path,&args);
+
+       PrintMessage(Debug,"Frame=%s",list[j]);
+
+       if(recursive_mode!=3)
          {
-          PrintMessage(Debug,"Link=%s",list[j]);
-
-          if(recursive_mode==1)
-             add_page(&pages,&npages,list[j]);
+          if(strcmp(page_host,link_host))
+             recurse=0;
           else
-            {
-             char *link_host=NULL,*link_path,*args;
-
-             SplitURL(list[j],&link_host,&link_path,&args);
-
-             if(!strcmp(page_host,link_host))
+             if(recursive_mode!=2)
                {
-                if(recursive_mode==0)
-                   add_page(&pages,&npages,list[j]);
-                else
-                  {
-                   char *end=page_path+strlen(page_path);
+                char *end=page_path+strlen(page_path);
 
-                   while(end>page_path)
-                      if(*end=='/')
-                         break;
-                      else
-                         end--;
-                   if(*end)
-                      *++end=0;
-                   if(end==page_path || !strncmp(page_path,link_path,end-page_path))
-                      add_page(&pages,&npages,list[j]);
-                  }
+                while(end>page_path)
+                   if(*end=='/')
+                      break;
+                   else
+                      end--;
+                if(*end)
+                   *++end=0;
+                if(end!=page_path && strncmp(page_path,link_path,end-page_path))
+                   recurse=0;
                }
+         }
+
+       if(recurse && status<300)
+         {
+          refresh=(char*)malloc(strlen(list[j])+64);
+
+          strcpy(refresh,"/refresh");
+          if(images)
+             strcat(refresh,"-images");
+          if(frames)
+             strcat(refresh,"-frames");
+          if(force)
+             strcat(refresh,"-force");
+          if(recursive_depth)
+            {
+             if(recursive_mode==1)
+                strcat(refresh,"-dir");
+             else if(recursive_mode==2)
+                strcat(refresh,"-host");
+             else /* recursive_mode==3 */
+                strcat(refresh,"-any");
+             sprintf(&refresh[strlen(refresh)],"-%d",recursive_depth);
+            }
+          if(!images && !frames && !recursive_depth && !force)
+             strcat(refresh,"-none");
+          strcat(refresh,"/");
+          strcat(refresh,link_host);
+          strcat(refresh,"/");
+          if(*link_path=='/')
+             strcat(refresh,link_path+1);
+          else if(*link_path)
+             strcat(refresh,link_path);
+          if(args)
+            {
+             strcat(refresh,"?");
+             strcat(refresh,args);
             }
          }
+       else
+          refresh=list[j];
+
+       new_request=RequestURL(refresh,url);
+       new_outgoing=OpenOutgoingSpoolFile(0);
+
+       if(new_outgoing==-1)
+          PrintMessage(Warning,"Cannot open the new outgoing request to write.");
+       else
+         {
+          write(new_outgoing,new_request,strlen(new_request));
+          CloseOutgoingSpoolFile(new_outgoing);
+          more=1;
+         }
+
+       if(refresh!=list[j])
+          free(refresh);
+       free(new_request);
       }
 
-    fclose(socket_f);
-    free(request);
-   }
+ if(status<400 && recursive_depth && parsed && (list=ListLinks()))
+    for(j=0;list[j];j++)
+      {
+       char *new_request,*refresh;
+       int new_outgoing;
+       char *link_host=NULL,*link_path,*args;
 
- free(buffer);
- for(i=0;i<npages;i++)
-    free(pages[i]);
- free(pages);
-}
+       SplitURL(list[j],&link_host,&link_path,&args);
 
+       PrintMessage(Debug,"Link=%s",list[j]);
 
-/*++++++++++++++++++++++++++++++++++++++
-  Add a page to the list of pages.
+       if(recursive_mode!=3)
+         {
+          if(strcmp(page_host,link_host))
+             continue;
+          else
+             if(recursive_mode!=2)
+               {
+                char *end=page_path+strlen(page_path);
 
-  char ***pages A pointer to the list of pages
+                while(end>page_path)
+                   if(*end=='/')
+                      break;
+                   else
+                      end--;
+                if(*end)
+                   *++end=0;
+                if(end!=page_path && strncmp(page_path,link_path,end-page_path))
+                   continue;
+               }
+         }
 
-  int *npages A pointer to the number of pages.
+       if(IsNotGotRecursive(link_host,link_path))
+          continue;
 
-  char *page The page to add.
-  ++++++++++++++++++++++++++++++++++++++*/
+       refresh=(char*)malloc(strlen(list[j])+64);
 
-static void add_page(char ***pages,int *npages,char *page)
-{
- int i;
+       strcpy(refresh,"/refresh");
+       if(images)
+          strcat(refresh,"-images");
+       if(frames)
+          strcat(refresh,"-frames");
+       if(force)
+          strcat(refresh,"-force");
+       if(recursive_depth-1)
+         {
+          if(recursive_mode==1)
+             strcat(refresh,"-dir");
+          else if(recursive_mode==2)
+             strcat(refresh,"-host");
+          else /* recursive_mode==3 */
+             strcat(refresh,"-any");
+          sprintf(&refresh[strlen(refresh)],"-%d",recursive_depth-1);
+         }
+       if(!images && !frames && !(recursive_depth-1) && !force)
+          strcat(refresh,"-none");
+       strcat(refresh,"/");
+       strcat(refresh,link_host);
+       strcat(refresh,"/");
+       if(*link_path=='/')
+          strcat(refresh,link_path+1);
+       else if(*link_path)
+          strcat(refresh,link_path);
+       if(args)
+         {
+          strcat(refresh,"?");
+          strcat(refresh,args);
+         }
 
- for(i=0;i<*npages;i++)
-    if(!strcmp((*pages)[i],page))
-       break;
+       new_request=RequestURL(refresh,url);
+       new_outgoing=OpenOutgoingSpoolFile(0);
 
- if(i==*npages)
-   {
-    if(!(*npages%8))
-       *pages=realloc(*pages,(8+*npages)*sizeof(char*));
+       if(new_outgoing==-1)
+          PrintMessage(Warning,"Cannot open the new outgoing request to write.");
+       else
+         {
+          write(new_outgoing,new_request,strlen(new_request));
+          CloseOutgoingSpoolFile(new_outgoing);
+          more=1;
+         }
 
-    (*pages)[*npages]=(char*)malloc(strlen(page)+1);
-    strcpy((*pages)[*npages],page);
-    (*npages)++;
-   }
+       free(refresh);
+       free(new_request);
+      }
+
+ return(more);
 }
