@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffle.c 1.17 1997/06/15 19:10:37 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffle.c 1.21 1997/09/07 09:32:11 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3.
   A user level program to interact with the server.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -50,7 +50,7 @@ Action;
 int main(int argc, char** argv)
 {
  int i;
- int force=0,images=0,recursive=0,recursive_mode=0;
+ int force=0,images=0,frames=0,recursive_depth=0,recursive_mode=0;
 
  Action action=None;
 
@@ -78,13 +78,13 @@ int main(int argc, char** argv)
        continue;
       }
 
-    if(!strcmp(argv[i],"-f"))
+    if(!strcmp(argv[i],"-F"))
       {
        if(action!=None && action!=Get)
          {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
        action=Get;
        if(force)
-         {fprintf(stderr,"wwwoffle: Only one '-f' argument may be given.\n"); exit(1);}
+         {fprintf(stderr,"wwwoffle: Only one '-F' argument may be given.\n"); exit(1);}
        force=1;
        argv[i]=NULL;
        continue;
@@ -102,25 +102,39 @@ int main(int argc, char** argv)
        continue;
       }
 
+    if(!strcmp(argv[i],"-f"))
+      {
+       if(action!=None && action!=Get)
+         {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
+       action=Get;
+       if(frames)
+         {fprintf(stderr,"wwwoffle: Only one '-f' argument may be given.\n"); exit(1);}
+       frames=1;
+       argv[i]=NULL;
+       continue;
+      }
+
     if(!strncmp(argv[i],"-R",2) || !strncmp(argv[i],"-r",2) || !strncmp(argv[i],"-d",2))
       {
        if(action!=None && action!=Get)
          {fprintf(stderr,"wwwoffle: Only one command at a time.\n\n");usage(0);}
        action=Get;
 
-       if(recursive)
-         {fprintf(stderr,"wwwoffle: Only one '-R', '-r' or '-d' argument may be given.\n"); exit(1);}
+       if(recursive_depth)
+         {fprintf(stderr,"wwwoffle: Only one '-d', '-r' or '-R' argument may be given.\n"); exit(1);}
 
-       if(argv[i][1]=='R')
+       if(argv[i][1]=='d')
           recursive_mode=1;
-       else if(argv[i][1]=='d')
-          recursive_mode=-1;
+       else if(argv[i][1]=='r')
+          recursive_mode=2;
+       else /* argv[i][1]=='R' */
+          recursive_mode=3;
 
        if(argv[i][2])
-          recursive=atoi(&argv[i][2]);
+          recursive_depth=atoi(&argv[i][2]);
        else
-          recursive=1;
-       if(recursive<=0)
+          recursive_depth=1;
+       if(recursive_depth<=0)
          {fprintf(stderr,"wwwoffle: The '-%c' argument may only be followed by a positive integer.\n",argv[i][1]); exit(1);}
 
        argv[i]=NULL;
@@ -226,7 +240,7 @@ int main(int argc, char** argv)
 
  /* Initialise things. */
 
- InitErrorHandler("wwwoffle",0);
+ InitErrorHandler("wwwoffle",0,1);
 
  if(ConfigFile)
    {
@@ -272,7 +286,10 @@ int main(int argc, char** argv)
     fflush(server_w);
 
     while((buffer=fgets_realloc(buffer,server_r)))
+      {
        fputs(buffer,stdout);
+       fflush(stdout);
+      }
 
     fclose(server_w);
     fclose(server_r);
@@ -287,19 +304,21 @@ int main(int argc, char** argv)
     strcpy(refresh,"refresh");
     if(images)
        strcat(refresh,"-images");
+    if(frames)
+       strcat(refresh,"-frames");
     if(force)
        strcat(refresh,"-force");
-    if(recursive)
+    if(recursive_depth)
       {
        if(recursive_mode==1)
-          strcat(refresh,"-any");
-       else if(recursive_mode==-1)
           strcat(refresh,"-dir");
-       else
+       else if(recursive_mode==2)
           strcat(refresh,"-host");
-       sprintf(&refresh[strlen(refresh)],"-%d",recursive);
+       else /* recursive_mode==3 */
+          strcat(refresh,"-any");
+       sprintf(&refresh[strlen(refresh)],"-%d",recursive_depth);
       }
-    if(!images && !recursive && !force)
+    if(!images && !frames && !recursive_depth && !force)
        strcat(refresh,"-none");
 
     for(i=1;i<argc;i++)
@@ -333,7 +352,10 @@ int main(int argc, char** argv)
           fflush(server_w);
 
           while((buffer=fgets_realloc(buffer,server_r)))
+            {
              fputs(buffer,stdout);
+             fflush(stdout);
+            }
 
           fclose(server_w);
           fclose(server_r);
@@ -425,7 +447,7 @@ int main(int argc, char** argv)
 static void usage(int verbose)
 {
  fputs("\n"
-       "WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b\n"
+       "WWWOFFLE - World Wide Web Offline Explorer - Version 1.3\n"
        "\n",stderr);
 
  if(verbose)
@@ -436,7 +458,7 @@ static void usage(int verbose)
  fputs("Usage: wwwoffle -h\n"
        "       wwwoffle -online | -offline | -fetch | -config | -purge\n"
        "       wwwoffle -o <url>\n"
-       "       wwwoffle [-i] [-f] [-(R|r|d)[<depth>]] <url> ...\n"
+       "       wwwoffle [-i] [-f] [-F] [-(d|r|R)[<depth>]] <url> ...\n"
        "\n"
        "Any of these can also take:  [-p <host>[:<port>] | -c <config-file>]\n"
        "\n",stderr);
@@ -462,12 +484,13 @@ static void usage(int verbose)
             " -o                  : Fetch the url and output it on the standard output.\n"
             "\n"
             " -i                  : Fetch the images included in the specified URLs.\n"
-            " -f                  : Force the url to be refreshed even if already cached.\n"
-            " -(R|r|d)[<depth>]   : Fetch pages linked to the URLs and their links,\n"
+            " -f                  : Fetch the frames included in the specified URLs.\n"
+            " -F                  : Force the url to be refreshed even if already cached.\n"
+            " -(d|r|R)[<depth>]   : Fetch pages linked to the URLs and their links,\n"
             "                       going no more than <depth> steps (default 1).\n"
-            "                        (-R => URLs on any host)\n"
-            "                        (-r => URLs on the same host)\n"
             "                        (-d => URLs in the same directory or sub-directory)\n"
+            "                        (-r => URLs on the same host)\n"
+            "                        (-R => URLs on any host)\n"
             "\n"
             " -p <host>[:<port>]  : The host name and port number to talk to the demon on.\n"
             "                       (Defaults to localhost for the server and\n"
