@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffle.h 1.23 1997/06/26 16:07:32 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffle.h 1.32 1997/09/23 18:08:52 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3.
   A header file for all of the programs wwwoffle, wwwoffled.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -18,6 +18,10 @@
 
 #include <stdio.h>
 
+
+/* ------------------------------------------------------------
+   The values in the section below can be changed if required, see INSTALL. */
+
 /*+ The default port number to use for the http proxy demon. +*/
 #define DEF_HTTP_PORT 8080
 
@@ -27,14 +31,21 @@
 /*+ The default spool directory. +*/
 #define DEF_SPOOL "/var/spool/wwwoffle"
 
+/*+ The default configuration file directory. +*/
+#define DEF_CONF "/var/spool/wwwoffle"
+
 /*+ The default age of pages in days before purging. +*/
 #define DEF_PURGE_AGE 14
 
-/*+ The maximum number of servers to fork in total. +*/
-#define MAX_SERVERS 8
+/*+ The absolute maximum number of servers to fork in total. +*/
+#define MAX_SERVERS 64
+/*+ The default total number of servers to fork. +*/
+#define DEF_MAX_SERVERS 8
 
-/*+ The maximum number of servers to fork for fetching previously requested pages. +*/
-#define MAX_FETCH_SERVERS 4
+/*+ The absolute maximum number of servers to fork for fetching previously requested pages. +*/
+#define MAX_FETCH_SERVERS 48
+/*+ The default number of servers to fork for fetching pages. +*/
+#define DEF_MAX_FETCH_SERVERS 4
 
 /*+ The character to use to replace '/' in the spool directories. +*/
 #define PATH_SEP '%'
@@ -42,6 +53,10 @@
 /*+ The time in seconds to wait while reading from a socket. +*/
 #define SOCKET_TIMEOUT 120
 
+/* The values in the section above can be changed if required, see INSTALL.
+   ------------------------------------------------------------ */
+
+
 /* In connect.c */
 
 void CommandConnect(FILE *client);
@@ -54,8 +69,9 @@ void PurgeCache(int fd);
 /* In spool.c */
 
 int OpenOutgoingSpoolFile(int rw);
-void CloseOutgoingSpoolFile(int fd);
-int OpenWebpageSpoolFile(int rw,char *host,char *path,char *args);
+char *CloseOutgoingSpoolFile(int fd);
+int DeleteOutgoingSpoolFile(char *name,char **url);
+int OpenWebpageSpoolFile(int rw,char **host,char *path,char *args);
 
 /* In parse.c */
 
@@ -68,18 +84,21 @@ char *ParseReply(int rep,int *status,int *length);
 void SplitURL(char *url,char **host,char **path,char **args);
 unsigned long MakeHashFromArgs(char *args);
 char *RFC822Date(long t,int utc);
-char *UrlDecode(char *str);
+long DateToTimeT(char *date);
+char *UrlDecode(char *str,int isform);
 char *UrlEncode(char *str);
+char* HTMLString(char* c);
 
 /* In messages.c */
 
 void ServerError(int fd,char *string);
 void RemoteHostError(int fd,char *url,char *reason);
 void CantRefreshPosted(int fd,char *url);
-void WillGetURL(int fd,char *url,int already);
+void WillGetURL(int fd,char *url,char *outgoing,int already);
 void RefreshRedirect(int fd,char *url);
 void WelcomePage(int fd,int password);
 void IllegalPage(int fd,char *path,char *args);
+void NotModified(int fd);
 
 /* In index.c */
 
@@ -87,7 +106,11 @@ void IndexPage(int fd,char *path,char *args);
 
 /* In control.c */
 
-void ControlPage(int fd,char *path,char *request);
+void ControlPage(int fd,char *path,char *args,char *request);
+
+/* In configedit.c */
+
+void ConfigEditPage(int fd,char *args,char *request);
 
 /* In refresh.c */
 
@@ -98,6 +121,7 @@ char *RefreshPage(int fd,char *path,char *args,char *url,char **request,int is_o
 int ParseHTML(FILE *file,char *host,char *path);
 char **ListImages(void);
 char **ListLinks(void);
+char **ListFrames(void);
 int GetHTMLEnd(void);
 
 /* In io.c */
