@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 1.16 1997/06/26 15:57:58 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffled.c 1.20 1997/09/07 09:35:23 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3.
   A demon program to maintain the database and spawn the servers.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -32,17 +32,17 @@
 
 
 static void usage(int verbose);
+static void demoninit(void);
+static void install_sighandlers(void);
 static void sigchild(int signum);
 static void sigexit(int signum);
+static void sighup(int signum);
 
 
 /*+ The server sockets that we listen on +*/
 int http_fd=-1,                 /*+ for the HTTP connections. +*/
     wwwoffle_fd=-1;             /*+ for the WWWOFFLE connections. +*/
 
-/*+ Set to 1 when the demon is to shutdown. +*/
-static int closedown=0;
-
 /*+ The online / offline status. +*/
 int online=0;
 
@@ -59,12 +59,17 @@ int fetch_pids[MAX_FETCH_SERVERS];
 /*+ The current status, fetching or not. +*/
 int fetching=0;
 
+/*+ Set to 1 when the demon is to shutdown. +*/
+static int closedown=0;
 
-/*+ The pid of a child that has exited. +*/
-static int exit_pid[MAX_SERVERS];
+/*+ Set to 1 when the demon is to re-read config file due to a sighup. +*/
+static int readconfig=0;
 
-/*+ The status of the child that has exited. +*/
-static int exit_status[MAX_SERVERS];
+/*+ Remember that we got a SIGCHLD +*/
+static sig_atomic_t got_sigchld=0;
+
+/*+ True if run as a demon +*/
+static int detached=1;
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -84,6 +89,12 @@ int main(int argc, char** argv)
     if(!strcmp(argv[i],"-h"))
        usage(1);
 
+    if(!strcmp(argv[i],"-d"))
+      {
+       detached=0;
+       continue;
+      }
+
     if(!strcmp(argv[i],"-c"))
       {
        if(++i>=argc)
@@ -101,17 +112,33 @@ int main(int argc, char** argv)
  for(i=0;i<MAX_FETCH_SERVERS;i++)
     fetch_pids[i]=0;
 
- for(i=0;i<MAX_SERVERS;i++)
-    exit_pid[i]=exit_status[i]=0;
-
- InitErrorHandler("wwwoffled",1);
+ InitErrorHandler("wwwoffled",1,!detached);
 
  if(ReadConfigFile(2))
     PrintMessage(Fatal,"Error in configuration file '%s'.",ConfigFile);
 
- PrintMessage(Important,"WWWOFFLE Demon Version 1.2b started");
+ PrintMessage(Important,"WWWOFFLE Demon Version 1.3 started");
  PrintMessage(Inform,"WWWOFFLE Read Configuration File.");
 
+ if(WWWOFFLE_Gid != -1)
+   {
+    if(setgid(WWWOFFLE_Gid)<0)
+       PrintMessage(Fatal,"Cannot set group id to %d [%!s].",WWWOFFLE_Gid);
+    if(setegid(WWWOFFLE_Gid)<0)
+       PrintMessage(Fatal,"Cannot set effective group id to %d [%!s].",WWWOFFLE_Gid);
+   }
+
+ if(WWWOFFLE_Uid != -1)
+   {
+    if(setuid(WWWOFFLE_Uid)<0)
+       PrintMessage(Fatal,"Cannot set user id to %d [%!s].",WWWOFFLE_Uid);
+    if(seteuid(WWWOFFLE_Uid)<0)
+       PrintMessage(Fatal,"Cannot set effective user id to %d [%!s].",WWWOFFLE_Uid);
+   }
+
+ if(WWWOFFLE_Gid != -1 || WWWOFFLE_Uid != -1)
+    PrintMessage(Inform,"Running with uid=%d, gid=%d.",getuid(),getgid());
+
  http_fd=OpenServerSocket(HTTP_Port);
  if(http_fd==-1)
     PrintMessage(Fatal,"Cannot create HTTP server socket.");
@@ -135,8 +162,14 @@ int main(int argc, char** argv)
  if(err==-1)
     PrintMessage(Fatal,"Cannot change to spool directory %s [%!s].",SpoolDir);
 
- sigchild(0);
- sigexit(0);
+ if(detached)
+   {
+    demoninit();
+    PrintMessage(Important,"Detached from terminal and changed pid to %d.",getpid());
+    InitErrorHandler("wwwoffled",1,!detached); /* pid changes after detaching. */
+   }
+
+ install_sighandlers();
 
  /* Loop around waiting for connections. */
 
@@ -156,7 +189,7 @@ int main(int argc, char** argv)
 
     FD_SET(wwwoffle_fd,&readfd);
 
-    if(n_servers<MAX_SERVERS)
+    if(n_servers<MaxServers)
        FD_SET(http_fd,&readfd);
 
     tv.tv_sec=10;
@@ -178,7 +211,7 @@ int main(int argc, char** argv)
                {
                 FILE *file=fdopen(client,"r");
 
-                PrintMessage(Important,"WWWOFFLE connection from %s.",host);
+                PrintMessage(Important,"WWWOFFLE Connection from %s.",host);
 
                 if(!file)
                   {
@@ -222,37 +255,86 @@ int main(int argc, char** argv)
          }
       }
 
-    for(i=0;i<MAX_SERVERS;i++)
-       if(exit_pid[i]!=0)
-         {
-          if(exit_status[i]>=0)
-             PrintMessage(Inform,"Child wwwoffles exited with status %d (pid=%d).",exit_status[i],exit_pid[i]);
-          else
-             PrintMessage(Important,"Child wwwoffles terminated by signal %d (pid=%d).",-exit_status[i],exit_pid[i]);
-          exit_pid[i]=exit_status[i]=0;
-         }
-
-    if(fetch_fd!=-1)
+    if(readconfig)
       {
-       if(!online)
-          fetching=0;
+       readconfig=0;
 
-       while(fetching && n_fetch_servers<MAX_FETCH_SERVERS && n_servers<MAX_SERVERS)
-          ForkServer(fetch_fd,0);
+       PrintMessage(Important,"SIGHUP signalled.");
 
-       if(!fetching && n_fetch_servers==0)
-         {
-          char *msg="WWWOFFLE No more to fetch.\n";
-          write(fetch_fd,msg,strlen(msg));
-          close(fetch_fd);
-          fetch_fd=-1;
+       PrintMessage(Important,"WWWOFFLE Re-reading Configuration File.");
 
-          PrintMessage(Important,"WWWOFFLE Fetch finished.");
-         }
+       if(ReadConfigFile(-1))
+          PrintMessage(Warning,"Error in configuration file; keeping old values.");
+
+       PrintMessage(Important,"WWWOFFLE Finished Re-reading Configuration File.");
+      }
+
+    if(got_sigchld)
+      {
+       int pid, status;
+
+       /* To avoid race conditions, reset the flag before fetching the status */
+       got_sigchld=0;
+
+       while((pid=waitpid(-1,&status,WNOHANG))>0)
+        {
+         int i;
+         int exitval=0;
+
+         if(WIFEXITED(status))
+            exitval=WEXITSTATUS(status);
+         else if(WIFSIGNALED(status))
+            exitval=-WTERMSIG(status);
+
+         if(exitval>=0)
+            PrintMessage(Inform,"Child wwwoffles exited with status %d (pid=%d).",exitval,pid);
+         else
+            PrintMessage(Important,"Child wwwoffles terminated by signal %d (pid=%d).",-exitval,pid);
+
+         n_servers--;
+
+         /* Check if the child that terminated is one of the fetching wwwoffles */
+         for(i=0;i<MaxFetchServers;i++)
+            if(fetch_pids[i]==pid)
+              {
+               n_fetch_servers--;
+               fetch_pids[i]=0;
+               break;
+              }
+
+         if(exitval==3)
+            fetching=0;
+
+         if(exitval==4 && online)
+            fetching=1;
+
+         if(!online)
+            fetching=0;
+        }
+
+        PrintMessage(Debug,"Currently running: %d servers total, %d fetchers.",n_servers,n_fetch_servers);
+      }
+
+    /* The select timed out or we got a signal. If we are currently fetching,
+       start fetch servers to look for jobs in the spool directory. */
+
+    while(fetching && n_fetch_servers<MaxFetchServers && n_servers<MaxServers)
+       ForkServer(fetch_fd,0);
+
+    if(fetch_fd!=-1 && !fetching && n_fetch_servers==0)
+      {
+       char *msg="WWWOFFLE No more to fetch.\n";
+       write(fetch_fd,msg,strlen(msg));
+       close(fetch_fd);
+       fetch_fd=-1;
+
+       PrintMessage(Important,"WWWOFFLE Fetch finished.");
       }
    }
  while(!closedown);
 
+ PrintMessage(Important,"Exit signalled.");
+
  /* Close down and exit. */
 
  CloseSocket(http_fd);
@@ -271,7 +353,7 @@ int main(int argc, char** argv)
 static void usage(int verbose)
 {
  fputs("\n"
-       "WWWOFFLED - World Wide Web Offline Explorer (Demon) - Version 1.2b\n"
+       "WWWOFFLED - World Wide Web Offline Explorer (Demon) - Version 1.3\n"
        "\n",stderr);
 
  if(verbose)
@@ -279,7 +361,7 @@ static void usage(int verbose)
           "                             [http://www.gedanken.demon.co.uk/]\n"
           "\n",stderr);
 
- fputs("Usage: wwwoffled [-h] [-c <config-file>]\n"
+ fputs("Usage: wwwoffled [-h] [-c <config-file>] [-d]\n"
        "\n",stderr);
 
  if(verbose)
@@ -288,12 +370,81 @@ static void usage(int verbose)
             "\n"
             "   -c <config-file>    : The name of the configuration file to use.\n"
             "                         (Default %s).\n"
+            "\n"
+            "   -d                  : Do not detach from the terminal and use stderr.\n"
             "\n",ConfigFile);
 
  exit(0);
 }
 
 
+/*++++++++++++++++++++++++++++++++++++++
+  Detach ourself from the controlling terminal.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void demoninit(void)
+{
+ pid_t pid=0;
+
+ pid=fork();
+ if(pid==-1)
+    PrintMessage(Fatal,"Cannot fork() to detach(1) [%!s].");
+ else if(pid)
+    exit(0);
+
+ setsid();
+
+ pid=fork();
+ if(pid==-1)
+    PrintMessage(Fatal,"Cannot fork() to detach(2) [%!s].");
+ else if(pid)
+    exit(0);
+
+ /* Close fds */
+ close(STDIN_FILENO);
+ close(STDOUT_FILENO);
+ close(STDERR_FILENO);
+}
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Install the signal handlers.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void install_sighandlers(void)
+{
+ struct sigaction action;
+
+ /* SIGCHLD */
+ action.sa_handler = sigchild;
+ sigemptyset (&action.sa_mask);
+ action.sa_flags = 0;
+ if(sigaction(SIGCHLD, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot install SIGCHLD handler.");
+
+ /* SIGINT, SIGQUIT, SIGTERM */
+ action.sa_handler = sigexit;
+ sigemptyset(&action.sa_mask);
+ sigaddset(&action.sa_mask, SIGINT);           /* Block all of them */
+ sigaddset(&action.sa_mask, SIGQUIT);
+ sigaddset(&action.sa_mask, SIGTERM);
+ action.sa_flags = 0;
+ if(sigaction(SIGINT, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot install SIGINT handler.");
+ if(sigaction(SIGQUIT, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot install SIGQUIT handler.");
+ if(sigaction(SIGTERM, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot install SIGTERM handler.");
+
+ /* SIGHUP */
+ action.sa_handler = sighup;
+ sigemptyset(&action.sa_mask);
+ action.sa_flags = 0;
+ if(sigaction(SIGHUP, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot install SIGHUP handler.");
+}
+
+
 /*++++++++++++++++++++++++++++++++++++++
   The signal handler for the child exiting.
 
@@ -302,60 +453,7 @@ static void usage(int verbose)
 
 static void sigchild(int signum)
 {
- void (*sig)();
- int pid;
- int status;
-
- if(signum)
-    while((pid=waitpid(-1,&status,WNOHANG))>0)
-      {
-       int i;
-       int exitval=0;
-
-       if(WIFEXITED(status))
-          exitval=WEXITSTATUS(status);
-       else if(WIFSIGNALED(status))
-          exitval=-WTERMSIG(status);
-
-       for(i=0;i<MAX_SERVERS;i++)
-          if(exit_pid[i]==0)
-            {
-             exit_pid[i]=pid;
-             exit_status[i]=exitval;
-             break;
-            }
-
-       n_servers--;
-
-       if(fetch_fd!=-1)
-         {
-          for(i=0;i<MAX_FETCH_SERVERS;i++)
-             if(fetch_pids[i]==pid)
-                break;
-
-          if(i!=MAX_FETCH_SERVERS)
-            {
-             n_fetch_servers--;
-             fetch_pids[i]=0;
-
-             if(exitval==3)
-                fetching=0;
-
-             if(exitval==4 && online)
-                fetching=1;
-            }
-         }
-      }
-
-#if defined(SIGCHLD)
- sig=signal(SIGCHLD,sigchild);
- if(sig==SIG_ERR)
-    PrintMessage(Fatal,"Cannot add SIGGHLD handler.");
-#else
- sig=signal(SIGCLD,sigchild);
- if(sig==SIG_ERR)
-    PrintMessage(Fatal,"Cannot add SIGGLD handler.");
-#endif
+ got_sigchld=1;
 }
 
 
@@ -367,25 +465,17 @@ static void sigchild(int signum)
 
 static void sigexit(int signum)
 {
- void (*sig)();
+ closedown=1;
+}
 
- if(signum)
-   {
-    closedown=1;
 
-    /* Ignore the PrintMessage re-entrancy problem since we are exiting anyway. */
-    PrintMessage(Important,"Exit signalled.");
-   }
+/*++++++++++++++++++++++++++++++++++++++
+  The signal handler for the signals to tell us to re-read the config file.
 
- sig=signal(SIGINT,sigexit);
- if(sig==SIG_ERR)
-    PrintMessage(Warning,"Cannot add SIGINT handler.\n");
+  int signum The signal number.
+  ++++++++++++++++++++++++++++++++++++++*/
 
- sig=signal(SIGQUIT,sigexit);
- if(sig==SIG_ERR)
-    PrintMessage(Warning,"Cannot add SIGQUIT handler.\n");
-
- sig=signal(SIGTERM,sigexit);
- if(sig==SIG_ERR)
-    PrintMessage(Warning,"Cannot add SIGTERM handler.\n");
+static void sighup(int signum)
+{
+ readconfig=1;
 }
