@@ -1,7 +1,7 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/RCS/wwwoffles.c 1.42 1997/06/26 16:07:32 amb Exp $
+  $Header: /home/amb/wwwoffle/RCS/wwwoffles.c 1.53 1997/10/06 18:07:10 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 1.2b.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 1.3a.
   A server to fetch the required pages.
   ******************/ /******************
   Written by Andrew M. Bishop
@@ -30,6 +30,8 @@
 #include "errors.h"
 
 
+static void uninstall_sighandlers(void);
+
 
 /*+ The mode of operation of the server. +*/
 typedef enum _Mode
@@ -64,7 +66,7 @@ int wwwoffles(int online,int browser,int client)
  int                     outgoing     =-1  , spool     =-1  ,server=-1;
  FILE *client_file=NULL,*outgoing_file=NULL,*spool_file=NULL;
 
- char *request=NULL,*reply=NULL,*url=NULL;
+ char *request=NULL,*reply=NULL,*url=NULL,*outgoing_filename=NULL;
  int replylen=0;
 
  Mode mode=None;
@@ -76,17 +78,9 @@ int wwwoffles(int online,int browser,int client)
 
  /* Initialise things. */
 
- InitErrorHandler("wwwoffles",1);
+ uninstall_sighandlers();
 
- signal(SIGPIPE,SIG_IGN);
- signal(SIGINT ,SIG_DFL);
- signal(SIGQUIT,SIG_DFL);
- signal(SIGTERM,SIG_DFL);
-#if defined(SIGCHLD)
- signal(SIGCHLD,SIG_DFL);
-#else
- signal(SIGCLD ,SIG_DFL);
-#endif
+ InitErrorHandler("wwwoffles",-1,-1); /* change name nothing else */
 
  if(online && browser)
     mode=Real;
@@ -99,12 +93,15 @@ int wwwoffles(int online,int browser,int client)
 
  /* Set up the client file. */
 
- if(client<0)
+ if(client<0 && mode!=Fetch)
    {PrintMessage(Warning,"Cannot use client file descriptor %d.",client);exit(1);}
 
- client_file=fdopen(client,"r");
- if(!client_file)
-   {PrintMessage(Warning,"Cannot use fdopen on the client file descriptor.");exit(1);}
+ if(client>=0)
+   {
+    client_file=fdopen(client,"r");
+    if(!client_file)
+      {PrintMessage(Warning,"Cannot use fdopen on the client file descriptor.");exit(1);}
+   }
 
  /* Set up the outgoing file. */
 
@@ -151,7 +148,8 @@ int wwwoffles(int online,int browser,int client)
     PrintMessage(Inform,"The host '%s' and/or path '%s' is on the list not to get.",host,path);
     exit(0);
    }
- else if(!*host)
+
+ if(!*host)
    {
     int newurl=0;
 
@@ -159,7 +157,9 @@ int wwwoffles(int online,int browser,int client)
       {
        url=RefreshPage(client,path,args,url,&request,online);
 
-       if(url)
+       if(url==(char*)1)
+          newurl=-1;
+       else if(url)
          {
           SplitURL(url,&host,&path,&args);
 
@@ -175,11 +175,11 @@ int wwwoffles(int online,int browser,int client)
           else if(mode==Real)
             {
              mode=RealRefresh;
-             OpenWebpageSpoolFile(-1,host,path,args);
+             OpenWebpageSpoolFile(-1,&host,path,args);
             }
           else if(mode==Spool)
             {
-             spool=OpenWebpageSpoolFile(1,host,path,args);
+             spool=OpenWebpageSpoolFile(1,&host,path,args);
              if(spool!=-1)
                {mode=SpoolRefresh;close(spool);}
              else
@@ -195,14 +195,17 @@ int wwwoffles(int online,int browser,int client)
     else if(!strncmp("index/",path,6))
        IndexPage(client,&path[6],args);
     else if(!strncmp("control/",path,8))
-       ControlPage(client,&path[8],request);
+       ControlPage(client,&path[8],args,request);
     else
        IllegalPage(client,path,args);
 
     if(!newurl)
        exit(0);
+    else if(newurl==-1)
+       exit(4);
    }
- else if(IsLocalNetHost(host))
+
+ if(IsLocalNetHost(host))
    {
     if(mode==Real || mode==Spool)
        mode=RealNoCache;
@@ -216,7 +219,8 @@ int wwwoffles(int online,int browser,int client)
    {
     mode=RealNoCache;
    }
- else if(!strncasecmp(request,"POST",4))
+
+ if(!strncasecmp(request,"POST",4))
    {
     if(mode==Spool)
        mode=SpoolGet;
@@ -231,11 +235,42 @@ int wwwoffles(int online,int browser,int client)
        exit(0);
       }
    }
- else if(strstr(request,"\nPragma: no-cache"))
+
+ if(strstr(request,"\nIf-Modified-Since:"))
    {
+    char *bol=strstr(request,"\nIf-Modified-Since:")+1,*eol=strchr(bol,'\n');
+    *eol++=0;
+
     if(mode==Spool)
       {
-       spool=OpenWebpageSpoolFile(1,host,path,args);
+       spool=OpenWebpageSpoolFile(1,&host,path,args);
+
+       if(spool!=-1)
+         {
+          struct stat buf;
+          long since;
+
+          buf.st_mtime=time(NULL)+1;
+          fstat(spool,&buf);
+          since=DateToTimeT(bol+19);
+
+          close(spool);
+
+          if(DateToTimeT(bol+19)>buf.st_mtime)
+            {NotModified(client);exit(0);}
+         }
+      }
+
+    while(*eol)
+       *bol++=*eol++;
+    *bol=0;
+   }
+
+ if(PragmaNoCache && strstr(request,"\nPragma: no-cache"))
+   {
+    if(mode==Spool || mode==SpoolGet)
+      {
+       spool=OpenWebpageSpoolFile(1,&host,path,args);
 
        if(spool==-1)
           mode=SpoolGet;
@@ -249,21 +284,21 @@ int wwwoffles(int online,int browser,int client)
    }
  else
    {
-    spool=OpenWebpageSpoolFile(1,host,path,args);
+    spool=OpenWebpageSpoolFile(1,&host,path,args);
 
     if(mode==Spool && spool==-1)
        mode=SpoolGet;
     else if(mode==Fetch && spool!=-1)
       {
        if(RequestChanges(spool,&request)==1)
-          OpenWebpageSpoolFile(-2,host,path,args);
+          OpenWebpageSpoolFile(-2,&host,path,args);
        else
           exit(0);
       }
     else if(mode==Real && spool!=-1)
       {
        if(RequestChanged && RequestChanges(spool,&request))
-          OpenWebpageSpoolFile(-2,host,path,args);
+          OpenWebpageSpoolFile(-2,&host,path,args);
        else
           mode=Spool;
       }
@@ -276,7 +311,7 @@ int wwwoffles(int online,int browser,int client)
 
  if(mode==Real || mode==Fetch || mode==SpoolGet)
    {
-    spool=OpenWebpageSpoolFile(0,host,path,args);
+    spool=OpenWebpageSpoolFile(0,&host,path,args);
 
     if(spool==-1)
       {
@@ -288,7 +323,7 @@ int wwwoffles(int online,int browser,int client)
    }
  else if(mode==Spool || mode==SpoolPragma)
    {
-    spool=OpenWebpageSpoolFile(1,host,path,args);
+    spool=OpenWebpageSpoolFile(1,&host,path,args);
 
     if(spool==-1)
       {
@@ -319,7 +354,7 @@ int wwwoffles(int online,int browser,int client)
     char *colon;
     char *proxy=WhichProxy(host);
 
-    if(proxy)
+    if(proxy && !IsLocalNetHost(host))
       {
        server_host=(char*)malloc(strlen(proxy)+1);
        strcpy(server_host,proxy);
@@ -416,7 +451,7 @@ int wwwoffles(int online,int browser,int client)
        PrintMessage(Warning,"Failed to write to remote host %s.",server_host);
        exit(1);
       }
-    else if(mode==Fetch)
+    else if(mode==Fetch && client!=-1)
       {
        char *buffer=(char*)malloc(strlen(url)+16);
        sprintf(buffer,"Fetching %s\n",url);
@@ -496,6 +531,13 @@ int wwwoffles(int online,int browser,int client)
  else
     reply=(char*)malloc(257);
 
+ /* Close the outgoing file if any. */
+
+ if(outgoing_file)
+    fclose(outgoing_file);
+ else if(outgoing>=0)
+    outgoing_filename=CloseOutgoingSpoolFile(outgoing);
+
  /* Read reply and process it. */
 
  if(mode==Real || mode==RealNoCache)
@@ -518,7 +560,7 @@ int wwwoffles(int online,int browser,int client)
           if(err==-1)
             {
              RemoteHostError(spool,url,"the connection was closed by the client during the transfer.");
-             PrintMessage(Warning,"Client disconnected during the transfer.");
+             PrintMessage(Warning,"Error writing to client [%!s]; client disconnected?.");
              exit(1);
             }
           else
@@ -560,34 +602,57 @@ int wwwoffles(int online,int browser,int client)
           ftruncate(spool,tell);
       }
 
-    if(FetchImages)
+    if(FetchImages || FetchFrames)
       {
-       char **images;
-
        lseek(spool,0,SEEK_SET);
        spool_file=fdopen(spool,"r");
        if(!spool_file)
          {PrintMessage(Warning,"Cannot use fdopen on the spool file descriptor.");exit(1);}
 
-       if(ParseHTML(spool_file,host,path) && (images=ListImages()))
-          for(n=0;images[n];n++)
-            {
-             char *new_request=RequestURL(images[n],url);
-             int new_outgoing=OpenOutgoingSpoolFile(0);
+       if(ParseHTML(spool_file,host,path))
+         {
+          char **list;
 
-             PrintMessage(Debug,"Image=%s",images[n]);
-
-             if(new_outgoing==-1)
-                PrintMessage(Warning,"Cannot open the new outgoing request to write.");
-             else
+          if(FetchImages && (list=ListImages()))
+             for(n=0;list[n];n++)
                {
-                write(new_outgoing,new_request,strlen(new_request));
-                CloseOutgoingSpoolFile(new_outgoing);
-                fetch_again++;
+                char *new_request=RequestURL(list[n],url);
+                int new_outgoing=OpenOutgoingSpoolFile(0);
+
+                PrintMessage(Debug,"Image=%s",list[n]);
+
+                if(new_outgoing==-1)
+                   PrintMessage(Warning,"Cannot open the new outgoing request to write.");
+                else
+                  {
+                   write(new_outgoing,new_request,strlen(new_request));
+                   CloseOutgoingSpoolFile(new_outgoing);
+                   fetch_again++;
+                  }
+
+                free(new_request);
                }
 
-             free(new_request);
-            }
+          if(FetchFrames && (list=ListFrames()))
+             for(n=0;list[n];n++)
+               {
+                char *new_request=RequestURL(list[n],url);
+                int new_outgoing=OpenOutgoingSpoolFile(0);
+
+                PrintMessage(Debug,"Frame=%s",list[n]);
+
+                if(new_outgoing==-1)
+                   PrintMessage(Warning,"Cannot open the new outgoing request to write.");
+                else
+                  {
+                   write(new_outgoing,new_request,strlen(new_request));
+                   CloseOutgoingSpoolFile(new_outgoing);
+                   fetch_again++;
+                  }
+
+                free(new_request);
+               }
+         }
       }
    }
  else if(mode==Spool || mode==SpoolPragma)
@@ -597,7 +662,7 @@ int wwwoffles(int online,int browser,int client)
     int n=read(spool,reply,64);
 
     if(!strncmp(reply,head,strlen(head)) || (!fstat(spool,&buf) && buf.st_size==0))
-       OpenWebpageSpoolFile(-1,host,path,args);
+       OpenWebpageSpoolFile(-1,&host,path,args);
 
     if(AddInfoRefresh)
       {
@@ -613,34 +678,52 @@ int wwwoffles(int online,int browser,int client)
 
        if((status>=200 && status<400) && ParseHTML(spool_file,host,path) && !fstat(spool,&buf))
          {
+          time_t t_ago=time(NULL)-buf.st_mtime;
           int position=GetHTMLEnd();
-          char *date=RFC822Date(buf.st_mtime,0),*encurl=UrlEncode(url),*localhost=GetLocalHost();
-          char *refresh=(char*)malloc(strlen(url)+2*strlen(localhost)+strlen(encurl)+128);
-          char *msg=(char*)malloc(strlen(date)+strlen(url)+2*strlen(localhost)+strlen(encurl)+256);
+          char *date=RFC822Date(buf.st_mtime,0),ago[16],*encurl=UrlEncode(url),*localhost=GetLocalHost();
+          char *refresh=(char*)malloc(strlen(url)+3*strlen(localhost)+2*strlen(encurl)+256);
+          char *msg=(char*)malloc(strlen(date)+strlen(url)+3*strlen(localhost)+2*strlen(encurl)+256);
+
+          if(t_ago<0)
+             sprintf(ago,"in the future");
+          else if(t_ago<3600)
+             sprintf(ago,"%ld min%s ago",t_ago/60,t_ago/60==1?"":"s");
+          else if(t_ago<(24*3600))
+             sprintf(ago,"%ld hour%s ago",t_ago/3600,t_ago/3600==1?"":"s");
+          else if(t_ago<(14*24*3600))
+             sprintf(ago,"%ld day%s ago",t_ago/(24*3600),t_ago/(24*3600)==1?"":"s");
+          else if(t_ago<(30*24*3600))
+             sprintf(ago,"%ld week%s ago",t_ago/(7*24*3600),t_ago/(7*24*3600)==1?"":"s");
+          else
+             sprintf(ago,"%ld month%s ago",t_ago/(30*24*3600),t_ago/(30*24*3600)==1?"":"s");
 
           if(!args)
-             sprintf(refresh,
-                     "[<a href=\"http://%s:%d/refresh/%s/%s\">Refresh</a>|"
-                     "<a href=\"http://%s:%d/refresh/?%s\">Options</a>]",
+             sprintf(refresh,"[<a href=\"http://%s:%d/control/delete?url=%s\">Delete</a>"
+                             "|<a href=\"http://%s:%d/refresh/%s/%s\">Refresh</a>|"
+                             "<a href=\"http://%s:%d/refresh/?%s\">Options</a>]",
+                     localhost,HTTP_Port,encurl,
                      localhost,HTTP_Port,host,path[0]=='/'?"":path,
                      localhost,HTTP_Port,encurl);
           else if(*args=='!')
-             sprintf(refresh,
-                     "[PostedNoRefresh]");
+             sprintf(refresh,"[<a href=\"http://%s:%d/control/delete?url=%s\">Delete</a>"
+                             "|Refresh|"
+                             "Opt]",
+                     localhost,HTTP_Port,encurl);
           else
-             sprintf(refresh,
-                     "[<a href=\"http://%s:%d/refresh/%s/%s?%s\">Refresh</a>|"
-                     "<a href=\"http://%s:%d/refresh/?%s\">Options</a>]",
+             sprintf(refresh,"[<a href=\"http://%s:%d/control/delete?url=%s\">Delete</a>"
+                             "|<a href=\"http://%s:%d/refresh/%s/%s?%s\">Refresh</a>|"
+                             "<a href=\"http://%s:%d/refresh/?%s\">Options</a>]",
+                     localhost,HTTP_Port,encurl,
                      localhost,HTTP_Port,host,path[0]=='/'?"":path,args,
                      localhost,HTTP_Port,encurl);
 
           sprintf(msg,"\n"
                       "<hr>\n"
                       "<p align=center>\n"
-                      "WWWOFFLE - %s - %s - WWWOFFLE\n"
+                      "WWWOFFLE - %s (%s) - %s - WWWOFFLE\n"
                       "</p>\n"
                       "<hr>\n",
-                  date,refresh);
+                  date,ago,refresh);
 
           fseek(spool_file,0,SEEK_SET);
           while((reply=fgets_realloc(reply,spool_file)))
@@ -690,8 +773,8 @@ int wwwoffles(int online,int browser,int client)
    {
     int tell;
 
-    WillGetURL(client,url,0);
-    WillGetURL(spool,url,1);
+    WillGetURL(client,url,outgoing_filename,0);
+    WillGetURL(spool,url,outgoing_filename,1);
 
     tell=lseek(spool,0,SEEK_CUR);
     if(tell!=-1)
@@ -707,11 +790,6 @@ int wwwoffles(int online,int browser,int client)
  if(server>=0)
     CloseSocket(server);
 
- if(outgoing_file)
-    fclose(outgoing_file);
- else if(outgoing>=0)
-    CloseOutgoingSpoolFile(outgoing);
-
  if(spool_file)
     fclose(spool_file);
  else if(spool>=0)
@@ -723,11 +801,55 @@ int wwwoffles(int online,int browser,int client)
  if(reply)
     free(reply);
 
- if(client>=0)
+ if(client_file)
     fclose(client_file);
+ else if(client>=0)
+    close(client);
 
  if(fetch_again)
     return(4);
  else
     return(0);
 }
+
+
+/*++++++++++++++++++++++++++++++++++++++
+  Uninstall the signal handlers.
+  ++++++++++++++++++++++++++++++++++++++*/
+
+static void uninstall_sighandlers(void)
+{
+ struct sigaction action;
+
+ /* SIGCHLD */
+ action.sa_handler = SIG_DFL;
+ sigemptyset (&action.sa_mask);
+ action.sa_flags = 0;
+ if(sigaction(SIGCHLD, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot uninstall SIGCHLD handler.");
+
+ /* SIGINT, SIGQUIT, SIGTERM */
+ action.sa_handler = SIG_DFL;
+ sigemptyset(&action.sa_mask);
+ action.sa_flags = 0;
+ if(sigaction(SIGINT, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot uninstall SIGINT handler.");
+ if(sigaction(SIGQUIT, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot uninstall SIGQUIT handler.");
+ if(sigaction(SIGTERM, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot uninstall SIGTERM handler.");
+
+ /* SIGHUP */
+ action.sa_handler = SIG_DFL;
+ sigemptyset(&action.sa_mask);
+ action.sa_flags = 0;
+ if(sigaction(SIGHUP, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot uninstall SIGHUP handler.");
+
+ /* SIGPIPE */
+ action.sa_handler = SIG_IGN;
+ sigemptyset (&action.sa_mask);
+ action.sa_flags = 0;
+ if(sigaction(SIGPIPE, &action, NULL) != 0)
+    PrintMessage(Warning, "Cannot ignore SIGPIPE.");
+}
